//****************************************************************************
// File: dialogs.c
//
// Purpose : Called by DoCommands() when want About dialog box or the
//           data validation dialog box.
//
// Functions:
//    AboutDlg()                - dialog function for about box
//    DataValDlgProc()          - dialog function for the data validation 
//                                dialog box
//    ValidateControlContents() - checks the contents of the specified 
//                                control.
//
// Development Team:
//    Sara Williams
//
// Written by Microsoft Product Support Services, Windows Developer Support
// Copyright (c) 1992 Microsoft Corporation. All rights reserved.
//****************************************************************************

#include <windows.h>
#include "global.h"

//****************************************************************************
// Function: AboutDlg
//    
// Purpose: Called by DoCommands() when want About dialog box.
//
// Parameters:
//    hDlg    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns: Depends on message.
//
// Comments:
//
// History:  Date       Author        Reason
//           2/1/92     saraw         Created
//****************************************************************************

BOOL FAR PASCAL AboutDlg (HWND hDlg,
                          unsigned message,
                          WORD wParam,
                          LONG lParam)
{
   switch (message)
   {
      case WM_INITDIALOG:
         return (TRUE);

      case WM_COMMAND:
         if ((wParam == IDOK) ||       // User clicked OK or Cancel?
             (wParam == IDCANCEL))     
         {
            EndDialog(hDlg, TRUE);     // Exits the dialog box 
            return (TRUE);
         }
         break;
   }

   return (FALSE); // Didn't process a message
}


//****************************************************************************
// Function: ValidateControlContents
//
// Purpose: Checks the validity of the contents of an edit control.
//
// Parameters:
//    hDlg    == Handle to the dialog box that contains the edit control
//    wParam  == identifier of the edit control to validate
//
// Returns: TRUE or FALSE
//
// Comments:
//
// History:  Date       Author        Reason
//           2/21/92    saraw         Created
//****************************************************************************
BOOL ValidateControlContents (HWND hDlg, WORD wParam)
{
   char szBuffer[128];
    int  iChars; // number of character in buffer
    int  i;      // loop control variable

   
    switch (wParam)
    {
      case ALPHAONLY:
      {
        // Check the contents of the edit control.  If any of the
        // characters are not alphabetic, then we return false,
        // otherwise, we return true.

        iChars = GetDlgItemText(hDlg, wParam, (LPSTR)szBuffer, 128);
          
        for (i = 0; i < iChars; i++)
        {
          if (!IsCharAlpha(szBuffer[i]))
             return FALSE;
        }
        return TRUE;
      }

      case NUMBERONLY:
      {
        // Check the contents of the edit control.  GetDlgItemInt does
        // all the work; it sets Success to false if it encounters a
        // non-numeric, so all we have to do is check Success and handle
        // the empty buffer case (GetDlgItemInt sets Success to FALSE on
        // an empty buffer, but we want to allow an empty buffer.

        BOOL Success;

        iChars = GetDlgItemText(hDlg, wParam, (LPSTR)szBuffer, 128);

        if (!iChars)    // empty buffer is okay
           return TRUE;

        GetDlgItemInt(hDlg, wParam, &Success, FALSE);

        if (!Success)    // this is an int 0 < x < 65356
           return FALSE;

        return TRUE;
      }

      default: 
        return TRUE;

    } // end switch

} // end ValidateControlContents()


//****************************************************************************
// Function: DataValDlgProc
//
// Purpose: Dialog box procedure for data validation example.
//
// Parameters:
//    hDlg    == Handle to _this_ window.
//    message == Message to process.
//    wParam  == WORD parameter -- depends on message
//    lParam  == LONG parameter -- depends on message
//
// Returns: TRUE or FALSE
//
// Comments:
//
// History:  Date       Author        Reason
//           2/1/92     saraw         Created
//****************************************************************************

BOOL FAR PASCAL DataValDlgProc (HWND hDlg,
                                unsigned message,
                                WORD wParam,
                                LONG lParam)
{
   static BOOL bCancel;
   static HWND hWndCancel;
   static BOOL bInIDOK;

   switch (message)
   {
      case WM_INITDIALOG:
         bCancel = FALSE;
         hWndCancel = GetDlgItem(hDlg, IDCANCEL);  // get the hWnd for the
                                                   // cancel button.
         SetFocus(GetDlgItem(hDlg, ALPHAONLY));
         return FALSE;  // we've set the focus, so the dlg manager doesn't
                        // have to worry about it.


      case WM_PARENTNOTIFY:  // We don't want to validate the contents of
                             // the control if the user has clicked the 
                             // cancel button.  

         // The lParam contains the x and y coordinates of the mouse when 
         // the button was pushed.  

         if ((wParam == WM_LBUTTONDOWN) && 
             (hWndCancel == ChildWindowFromPoint(hDlg, MAKEPOINT(lParam))))
            bCancel = TRUE;

         break;

      case WM_COMMAND:
      {
         if (wParam == IDOK)
         {    
             BOOL bAValid = TRUE;
             BOOL bNValid = TRUE;

             // you would save the information the user entered in the
             // edit controls at this point.
             
             // We need to validate control contents here since the 
             // use could have hit enter from one of the edit controls, and
             // the dialog box gets this message before it gets the
             // EN_KILLFOCUS for the edit control.

             // By definition, at most one of the controls can be invalid.
             // So, we check them, end the dialog if they are both valid,
             // or send a WM_NOTVALID to the invalid control.

             if ((bAValid = ValidateControlContents(hDlg, ALPHAONLY)) && 
                 (bNValid = ValidateControlContents(hDlg, NUMBERONLY)))
             {
               EndDialog(hDlg, TRUE);
               return TRUE;
             }

             // Using this boolean prevents multiple WM_NOTVALID
             // messages from being sent.
             bInIDOK = TRUE;

             if (bAValid)  // bNValid must be false
                SendMessage(hDlg, WM_NOTVALID, NUMBERONLY, 
                            MAKELONG(GetDlgItem(hDlg, NUMBERONLY), 0));
             
             else // bAValid is false
                SendMessage(hDlg, WM_NOTVALID, ALPHAONLY, 
                            MAKELONG(GetDlgItem(hDlg, ALPHAONLY),0));
             
             bInIDOK = FALSE;
             break;

         }

         if (wParam == IDCANCEL)      // User wants to get out
         {
            EndDialog(hDlg, FALSE);
            return TRUE;
         }



           // We check to make sure that this message was not received as 
           // part of the IDOK processing so that multiple WM_NOTVALID
           // messages are not received.

         if (!bInIDOK && HIWORD(lParam) == EN_KILLFOCUS)  // focus is changing
         {
            // check validity of control and post the WM_NOTVALID message
            // if it is invalid
            if ((!bCancel) && (!ValidateControlContents(hDlg, wParam)))
              PostMessage(hDlg, WM_NOTVALID, wParam, lParam); 

            return TRUE;

         }
         break;

      }  // end case WM_COMMAND


      case WM_NOTVALID:  // one of the edit controls is not valid.
                         // Tell the user and set the focus back to the
                         // invalid control.
      {

         if (wParam == ALPHAONLY)
            MessageBox(hDlg, "Please enter letters only.", 
                       "Invalid entry", MB_OK);

         else // if (wParam == NUMBERONLY)
            MessageBox(hDlg, "Please enter an integer less than 65536.", 
                       "Invalid entry", MB_OK);

         SetFocus(LOWORD(lParam));

           // Highlight the control's text.
         SendMessage(LOWORD(lParam), EM_SETSEL, 0, MAKELONG(0, 0x7fff));
         
         return TRUE;
      }

   }

   return (FALSE); // Didn't process a message
}
                                          
