#include "parse.h"

/*
 	File: Display.c

	Purpose: Contains all the display functions for the various allocations
             That are performed

	Calling Convention: All functions return nothing (void)
                        All are called in the same manner: 
						display_<requirement> (BYTE *data,ULONG nCur,UINT nLen)

	Entry:	data -> A byte near pointer to the data stream read from the card.
            nCur -> The offset into said data stream
            Len  -> The lenght of the current data item.

	Side Effects: None
  */

void
display_Log_Dev_ID(BYTE *data,ULONG nCur,UINT Len) 
{

	ULONG DevID;
	BYTE  Flags1,Flags2;
	int i;

	//The Device ID is comprised of the first four words of the Field
    DevID=*((ULONG *)&(data[nCur]));
	nCur+=4;

	LogPrintf("Device ID         : 0x%08lX\n",DevID);

	if (data[nCur] & 1) {
		LogPrintf("Logical Device is required for boot\n");
	}

	//Flags 1 & 2 aren't strictly neccessary, they just make it clearer
	Flags1=data[nCur];	
	Flags2=data[nCur+1];

	if (Flags1 & 2) {
		LogPrintf("Supports I/O Range Checking\n");
	}

	//Mask off the bottom bit
	Flags1 &= 0xfe;

	LogPrintf("Vendor Defined Logical Device Control Registers:\n");
	
	for (i=2;i<0x8;i++) {
		if ((Flags1 >> i) & 0x01) {
			LogPrintf("!ERROR! -Caution- Register 0x3%1X is reserved but reports command support\n",i);
		}
	}

	if (Flags2) {	

		for (i=8;i<0x10;i++) {
			if ((Flags2 >> i-8) & 0x01) {
			LogPrintf("0x3%1X ",i);
			}
		}
		LogPrintf("\n");
	} else {
		LogPrintf("None\n\n");
	}
}


void
display_IRQ_FMT(BYTE *data,ULONG nCur,UINT Len) 
{

	int i;


	LogPrintf("\nIRQ lines supported:\n");

	for (i=0;i<0x8;i++) {
		if ((data[nCur] >> i) & 0x01) {
			LogPrintf("0x%X ",i);
		}
	}
	for (i=8;i<0x10;i++) {
		if ((data[nCur+1] >> i-8) & 0x01) {
			LogPrintf("0x%X ",i);
		}
	}
	LogPrintf("\n");


	if (Len ==2 ) {
		return;
	}

	nCur+=2;
	//Bits 4-7 are reserved and MUST be 0
	if (data[nCur] >> 4) {
		LogPrintf("!ERROR! Reserved bits set in IRQ format field, not processing characteristics\n");
		return;
	}


	//Display the IRQ types
	LogPrintf("IRQ type: \n");
	if ((data[nCur] >> 3) & 1){
		LogPrintf("Low True Level Sensitive\n");
	}
	if ((data[nCur] >> 2) & 1) {
		LogPrintf("Low True Level Sensitive\n");
	}
	if ((data[nCur] >> 1) & 1) { 
		LogPrintf("Low True Level Sensitive\n");
	}
	if (data[nCur] & 1) {
		LogPrintf("High True Edge Sensitive\n");
	}
	LogPrintf("\n");
}

	
void
display_DMA_FMT(BYTE *data,ULONG nCur,UINT Len) 
{

	int i;	

	//Print out the channel mask
	LogPrintf("DMA Channels supported: \n");
	for (i=0;i<8;i++) {
		if ((data[nCur] >> i) & 1 ) {
			LogPrintf("%d ",i);
		}
	}
	LogPrintf("\n");
	nCur++;


	//Channel Characteristics
	LogPrintf("\nDMA Channel Characteristics:\n");

	//TransferType
	if (!(data[nCur] & 3)) {
		LogPrintf("8 bit only\n");
	} 
	else {
		if ((data[nCur] & 3)==1 )  { 
			LogPrintf("8 and 16 bit\n");
		} 
		else {
			if ((data[nCur] & 3)==2 )  { 
				LogPrintf("16 bit only\n");
			}
			else {
				LogPrintf("!ERROR! Reserved Channel type\n");
			}

		}
	}

	//Bus Master Status: if set, yes, else no.
	LogPrintf("Logical device is %sa bus master\n",((data[nCur] >> 2) & 1)?"":"not ");

	//DMA Byte Mode: if set, yes, else no
	LogPrintf("DMA may %sexecute in count by byte mode\n",((data[nCur] >> 3) & 1)?"":"not ");

	//DMA Word Mode: if set, yes, else no
	LogPrintf("DMA may %sexecute in count by word mode\n",((data[nCur] >> 4) & 1)?"":"not ");


	//Channel Speed Supported
	if ((!data[nCur] >> 5) & 3) {
		LogPrintf("Compatibility mode\n");
	}
	else {
		if (((data[nCur] >> 5) & 3)==1) {
			LogPrintf("Type A DMA (as described in EISA Spec.)\n");
		}
		else {
			if (((data[nCur] >> 5) & 3)==2) {
				LogPrintf("Type B DMA\n");
			}
			else {
				LogPrintf("Type F DMA\n");
			}
		}
	}

	//Reserved
	if ((data[nCur] >>7 ) &1 ) {
		LogPrintf("!ERROR! Reserved bit set!\n");
	}

	LogPrintf("\n");
}



void
display_IO_DESC(BYTE *data,ULONG nCur,UINT Len)
{

	WORD address,minimum;

	LogPrintf("\nI/O Descriptors: \n");
	//Decoding:
	if (data[nCur] & 1) {
		LogPrintf("Card decodes full 16-bit ISA Address\n");
	}
	else {
		LogPrintf("Card only decodes ISA address bits 0-9\n");
	}

	//Reserved:
	if (data[nCur] >> 1) {
		LogPrintf("!ERROR! Reserved bits set in I/O Port Descriptor Information Byte\n");
	}
	nCur++;

	//Range minimum Address
	minimum=data[nCur]+(data[nCur+1] << 8);

	LogPrintf("Minimum base I/O Address: 0x%X\n",minimum);

	//Range maximum Address
	nCur+=2;
	address=data[nCur]+(data[nCur+1] << 8);
	LogPrintf("Maximum base I/O Address: 0x%X\n",address);

	nCur+=2;
	//Alignment, mask out higher word
	LogPrintf("Base I/O Address requires 0x%X byte alignment\n",data[nCur] & 0xff);

	nCur++;
	//Range, mask out higer word
	LogPrintf("Device requires %d Continuous ports\n",data[nCur] & 0xff);

	if ((address-minimum) % (data[nCur-1] & 0xff ) ){
		LogPrintf("!ERROR! - Caution - Addressing space is NOT a multiple of step size!\n");
	}

	LogPrintf("\n");

}


void
display_Fixed_DESC(BYTE *data,ULONG nCur,UINT Len)
{

	WORD address;
	
	LogPrintf("\nFixed I/O Descriptors: \n");
	address=data[nCur]+(data[nCur+1] << 8);

	LogPrintf("Base Address: 0x%X\n",address);

	nCur+=2;

	//Range, mask out higer word
	LogPrintf("Port requires %d Continuous ports\n",data[nCur] & 0xff);

}


void 
display_Mem_RD(BYTE *data,ULONG nCur,UINT Len)
{

	DWORD maximum,minimum;
	WORD length,align;

	LogPrintf("\nMemory Range Information:\n");

	//Information Byte:
	if (data[nCur] & 1) {
		LogPrintf("Writeable Memory\n");
	}
	else {
		LogPrintf("Non Writeable Memory (ROM)\n");
	}

	if (data[nCur] & 2) {
		LogPrintf("Read Cacheable, write through\n");
	}
	else {
		LogPrintf("Non-Cacheable\n");
	}

	if (data[nCur] & 4) {
		LogPrintf("Decode Supports High Address\n");
	}
	else {
		LogPrintf("Decode Supports Extent\n");
	}

	//Shadowable, and expansion ROM checks
	LogPrintf("Memory is %sshadowable\n",(data[nCur] & 32)?"":"not ");
	LogPrintf("Memory is %san expansion ROM\n",(data[nCur] & 64)?"":"not ");


	//Memory Control
	if (!((data[nCur] >> 3) & 3)) {
		LogPrintf("8 bit memory only\n");
	} 
	else {
		if (((data[nCur] >> 3) & 3)==1 )  { 
			LogPrintf("8 and 16 bit\n");
		} 
		else {
			if (((data[nCur] >> 3) & 3)==2 )  { 
				LogPrintf("16 bit memory only\n");
			}
			else {
				LogPrintf("!ERROR! Reserved Memory Control type\n");
			}

		}
	}

	nCur++;

	//Minimum Base Address
	minimum=data[nCur]+(data[nCur+1] << 8);

	//Maximum Base Address
	nCur+=2;
	maximum=data[nCur]+(data[nCur+1] << 8);

	//Base Alignment
	nCur+=2;
	align=data[nCur]+(data[nCur+1] << 8);

	//Range Length
	nCur+=2;
	length=data[nCur]+(data[nCur+1] << 8);

	
	LogPrintf("Minimum base Memory Address: 0x%08lX\n",minimum*256);
	LogPrintf("Maximum base Memory Address: 0x%08lX\n",maximum*256);
	//Alignment, if 0 then 64K alignment.
	LogPrintf("Base Address Increment:      0x%08X\n",(align)?align:0x10000);

	//Memory range required, in 256 byte blocks
	LogPrintf("Memory range required:       0x%08X bytes\n",length*256);


}


