//***************************************************************************
//  Module:   DRVRAPP.C
//
//  Purpose:
//      Simple Windows application which provides graphical interface
//      to loading/unloading INSTDRVR.DLL, the associated installable
//      driver.
//
//  Description of functions:
//      WinMain()   Registers class, creates main window, message pump.
//      WndProc()   Main overlapped window (actually a modeless dialog).
//      About()     Yer standard about box dialog.
//---------------------------------------------------------------------------
//  Copyright (c) 1993 Microsoft Corporation.  All Rights Reserved.
//***************************************************************************


#include <windows.h>
#include <stdlib.h>
#include "DrvrApp.h"            // specific to this program
#include "resapp.h"             // resource header generated by AppStudio

extern const HINSTANCE _hInstance;      // supplied by C RunTime startup code 
extern const HINSTANCE _hPrevInstance;  // ditto      

char szAppName [] = "DrvrApp"; 
char szDriverName[] = "instdrvr.dll";
char szTmp[80] = "";    // scratch pad buffer
static unsigned int uDriverHandleCount = 0;


//************************************************************************
//  int PASCAL WinMain 
//
//  Description:
//      An extremely simple WinMain -- replaces the normal sequence of 
//      registering a class, creating a window, and pumping messages with
//      a super-simple call to DialogBox.  Easy as pie!
//
//  Returns:
//      Success:  TRUE
//      Failure:  FALSE
//************************************************************************

int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPSTR lpszCmdLine, int nCmdShow)
{
    if (DialogBox(_hInstance, MAKEINTRESOURCE(IDD_MAINWINDOW), 0,
            WndProc) == -1)
    {
        MessageBox(0, "Could not create main window", "DrvrApp", MB_OK);
        return FALSE; 
    }    
    return TRUE;    
} // end of WinMain()


//************************************************************************
//  LRESULT CALLBACK _export WndProc 
//
//  Description:
//      It looks like a standard window procedure, but it's actually
//      a dialog box procedure.  The dialog controls result in the various
//      WM_COMMAND messages which this function handles.  Everything else
//      is uninteresting, so we let the dialog manager handle it.
//  
//      When the user clicks "Open Driver", we call INSTDRVR.DLL's OpenDriver
//      API and add the driver instance handle to the listbox.
//
//      When the user clicks "Close Driver", we call INSTDRVR.DLL's CloseDriver
//      API on the driver instance handle retrieved from the list box.
//
//  Returns:
//       TRUE or FALSE to indicate to dialog manager whether we handled msg
//************************************************************************

BOOL CALLBACK _export WndProc (HWND hWnd, UINT message, WPARAM wParam,
                                                        LPARAM lParam)
{  
    switch (message)
    {   
    case WM_INITDIALOG:
        { 
        HDRVR hDriver;
        LRESULT lResult;
            
        hDriver = OpenDriver(szDriverName, NULL, NULL); 
        if (hDriver == NULL) 
        {
            MessageBox(0, "Couldn't open driver", szAppName, MB_OK);
            return TRUE;
        }
        // find out if the driver can be configured; if not, disable the
        // configure dialog button.  
        lResult = SendDriverMessage(hDriver, DRV_QUERYCONFIGURE, NULL, NULL);
        if (lResult == 0)
            EnableWindow(GetDlgItem(hWnd, IDC_CONFIGURE), FALSE);
        if (CloseDriver(hDriver, NULL, NULL) == 0)
            MessageBox(0, "Couldn't close driver", szAppName, MB_OK);
        }
        return TRUE;

    case WM_COMMAND:
        switch (wParam)
        {   
        HDRVR hDriver;
        DWORD dwIndex;
        
        case IDC_OPENDRIVER:
            hDriver = OpenDriver(szDriverName, NULL, NULL); 
            if (hDriver == NULL) 
            {
                MessageBox(0, "Couldn't open driver", szAppName, MB_OK);
                break;
            }    
            if (uDriverHandleCount == 0)
                EnableWindow(GetDlgItem(hWnd, IDC_CLOSEDRIVER), TRUE);
            uDriverHandleCount++;
            // Add the handle to the listbox...
            itoa(hDriver, szTmp, 10);
            dwIndex = SendDlgItemMessage(hWnd, IDC_DRVRHANDLES, LB_ADDSTRING,
                0, (LPARAM) (LPCSTR) szTmp); 
            SendDlgItemMessage(hWnd, IDC_DRVRHANDLES, LB_SETCURSEL, 
                (WPARAM) dwIndex, 0L);
            break;
            
        case IDC_CLOSEDRIVER:
            dwIndex = SendDlgItemMessage(hWnd, IDC_DRVRHANDLES, 
                LB_GETCURSEL, 0, 0L);
            if (dwIndex == LB_ERR)
                break;
            SendDlgItemMessage(hWnd, IDC_DRVRHANDLES, LB_GETTEXT,
                (WPARAM) dwIndex, (LPARAM) (LPCSTR) szTmp);
            hDriver = atoi(szTmp);
            if (CloseDriver(hDriver, NULL, NULL) == 0)
            {
                MessageBox(0, "Couldn't close driver", szAppName, MB_OK);
                break;
            }
            SendDlgItemMessage(hWnd, IDC_DRVRHANDLES, LB_DELETESTRING,
                (WPARAM) dwIndex, 0L);
            SendDlgItemMessage(hWnd, IDC_DRVRHANDLES, LB_SETCURSEL,
                (WPARAM) 0, 0L);
            uDriverHandleCount--;
            if (uDriverHandleCount == 0)
                EnableWindow(GetDlgItem(hWnd, IDC_CLOSEDRIVER), FALSE);
            break;
            
        case IDC_CONFIGURE:
            {
            LRESULT lResult;
            
            hDriver = OpenDriver(szDriverName, NULL, NULL); 
            if (hDriver == NULL) 
            {
                MessageBox(0, "Couldn't open driver", szAppName, MB_OK);
                break;
            }
            lResult = SendDriverMessage(hDriver, DRV_CONFIGURE, NULL, NULL);
            if (CloseDriver(hDriver, NULL, NULL) == 0)
                MessageBox(0, "Couldn't close driver", szAppName, MB_OK);
            }
            break;
            
        case IDC_EXIT:
            EndDialog(hWnd, TRUE);
            break; 

        case IDC_ABOUT:
            DialogBox(_hInstance, MAKEINTRESOURCE(IDD_ABOUT), hWnd, About);
            break; 
        } // end switch 
        return TRUE;  // we handled the message
    }
    return FALSE;  // we didn't handle the message
} // end of WndProc()
    


//************************************************************************
//  BOOL CALLBACK _export About
//
//  Description:
//      Just yer standard About box.  Boring.
//************************************************************************

BOOL CALLBACK _export About(HWND hDlg, UINT message, WPARAM wParam, 
                                                     LPARAM lParam)
{
    switch (message) 
    {
    case WM_COMMAND:
        if ((wParam == IDOK) || (wParam == IDCANCEL))
        {
            EndDialog(hDlg, TRUE); 
            return TRUE; 
        }
        break;
    }
    return FALSE; 
} // end of About()
   
        

//***************************************************************************
//  End of File: drvrapp.c
//***************************************************************************

