//*************************************************************
//  File name: INSTDRVR.C
//
//  Purpose:
//      Contains the DriverProc for the installable driver.  
//      This code calls all the appropriate handler based upon 
//      the message received.
//
//  Description of functions:
//      DriverProc()            The driver's equivalent of a window proc.
//      DebugDriverMessage()    Utility function for outputting debug msgs.
//      ConfigureDialog()       Dialog box procedure called in DRV_CONFIGURE.
//      LibMain()               Library initialization function.
//      _WEP()                  MS C7 version of WEP.   
//      UpdateBootDrivers()     Add/Delete driver from "drivers=" line in [boot]
//*************************************************************

#include <windows.h>
#include <string.h>
#include "instdrvr.h" 
#include "resdll.h" 
        
#define TMP_SIZE 80
        
extern const HINSTANCE _hModule;      // supplied by C RunTime startup code 
            
UINT fDebugMsgs = 0;    // boolean flag: should we output debug messages?
static char szTmp[TMP_SIZE];    // scratch pad; declared static to reduce
                                // stack usage (we can't control what stack
                                // we run on.
static char szDriverName[] = "InstDrvr";
static char szDriverFileName[] = "instdrvr.dll";

//*************************************************************
//  LRESULT CALLBACK DriverProc
//
//  Purpose:  Entry point for the installable device driver
//
//  Return: DefDriverProc or a status if we handle the message
//          ourselves
//*************************************************************

LRESULT CALLBACK _export DriverProc (DWORD   dwDriverIdentifier,
                              HDRVR   hDriver,
                              UINT    wMessage,
                              LPARAM  lParam1,
                              LPARAM  lParam2)
{
    DWORD dwRes = 0L;

    switch (wMessage)
    {
    case DRV_LOAD:
        // Sent when the driver is loaded. This is always   
        // the first message received by a driver.

        // Find out if we're supposed to output debug strings.
        // If no string is found (as when first installing driver),
        // default to "0" so we don't force debug msgs on unwilling users.
        GetPrivateProfileString(szDriverName,   //section
            "DebugMsgs",                        // entry
            "0",                                // default
            szTmp,                              // dest
            TMP_SIZE,                           // count
            "system.ini");                      // file
        fDebugMsgs = (szTmp[0] != '0');

        DebugDriverMessage("DRV_LOAD\t", lParam1, lParam2);
        dwRes = 1L;     // Return 0L to fail.               
        break;

    case DRV_FREE:
        // Sent when the driver is about to be discarded.   
        // This is the last message a driver receives       
        // before it is freed.                              
        DebugDriverMessage("DRV_FREE\t", lParam1, lParam2);
        dwRes = 1L;     // Return value is ignored.         
        break;

    case DRV_OPEN:
        // Sent when the driver is opened.
        DebugDriverMessage("DRV_OPEN\t", lParam1, lParam2);
        dwRes = 1L;     // Return 0L to fail.               
                        // This value is subsequently used  
        break;          // for dwDriverIdentifier.          

    case DRV_CLOSE:
        // Sent when the driver is closed. Drivers are      
        // unloaded when the open count reaches zero.
        DebugDriverMessage("DRV_CLOSE\t", lParam1, lParam2);
        dwRes = 1L;     // Return 0L to fail.               
        break;

    case DRV_ENABLE:
        // Sent when the driver is loaded or reloaded and   
        // when Windows is enabled. Hook or rehook          
        // interrupts and initialize hardware. Expect the   
        // driver to be in memory only between the enable   
        // and disable messages.                            
        DebugDriverMessage("DRV_ENABLE\t", lParam1, lParam2);
        dwRes = 1L;     // Return value is ignored.         
        break;

    case DRV_DISABLE:
        // Sent before the driver is freed or when Windows  
        // is disabled. Unhook interrupts and place         
        // peripherals in an inactive state.                
        DebugDriverMessage("DRV_DISABLE\t", lParam1, lParam2);
        dwRes = 1L;     // Return value is ignored.         
        break;

    case DRV_INSTALL:
        // Sent when the driver is installed.               
        DebugDriverMessage("DRV_INSTALL\t", lParam1, lParam2);
        // Create an [InstDrvr] section in SYSTEM.INI containing
        // default settings: DebugMsgs=0 and LoadAtBoot=0.
        WritePrivateProfileString(szDriverName,         // section
            "DebugMsgs",                                // entry
            "0",                                        // value
            "system.ini");                              // file
        WritePrivateProfileString(szDriverName,         // section
            "LoadAtBoot",                               // entry
            "0",                                        // value
            "system.ini");                              // file
        dwRes = DRVCNF_OK; // Can also return DRVCNF_CANCEL       
                           // or DRVCNF_RESTART.                 
        break;

    case DRV_REMOVE:
        // Sent when the driver is removed.                 
        DebugDriverMessage("DRV_REMOVE\t", lParam1, lParam2);  
        // Because we created an [InstDrvr] section when we were installed,
        // it's our job to delete this section when we get removed...
        WritePrivateProfileString(  szDriverName,   // section
                                    NULL,           // delete entire section
                                    NULL,           // NA
                                    "system.ini");  // file
        UpdateBootDrivers(FALSE);  // remove driver 

        dwRes = 0L;     // Return value is ignored.         
        break;

    case DRV_QUERYCONFIGURE:
        // Sent to determine if the driver can be           
        // configured.  This driver can be configured, so it returns non-zero
        DebugDriverMessage("DRV_QUERYCONFIGURE", lParam1, lParam2);
        dwRes = 1L;     // 1 indicates configuration dialog supported.
        break;

    case DRV_CONFIGURE:
        // Sent to display the custom-configuration         
        // dialog box for the driver.  We call ConfigureDialog
        DebugDriverMessage("DRV_CONFIGURE\t", lParam1, lParam2);
        dwRes = DialogBox(_hModule, MAKEINTRESOURCE(IDD_CONFIGURE), 
            0, ConfigureDialog);
        break;  
        
    case DRV_POWER:
        // Notifies driver that power source for device
        // is about to be turned off or on.
        DebugDriverMessage("DRV_POWER\t", lParam1, lParam2);
        dwRes = 1L;
        break;

    case DRV_EXITSESSION:
        // Sent when Windows is terminating
        DebugDriverMessage("DRV_EXITSESSION", lParam1, lParam2);
        break;       
        
    case DRV_EXITAPPLICATION:
        // Notifies driver that an application has terminated
        // (lParam1 specifies normal or abnormal exit)
        DebugDriverMessage("DRV_EXITAPPLICATION", lParam1, lParam2);
        break;
                
    default:
        // Process any messages not explicitly trapped.
        {
            static char szMsg[32];
            
            wsprintf(szMsg, "Untrapped message: %x", wMessage);
            DebugDriverMessage(szMsg, lParam1, lParam2);
        }
        return DefDriverProc (dwDriverIdentifier, hDriver,
                             wMessage, lParam1, lParam2);
    } // end switch()
    return dwRes;
}



//*************************************************************
//  void DebugDriverMessage
//
//  Purpose:    Utility function for displaying debug strings from
//              the installable device driver
//
//  Comment:    If debugging messages are enabled, it calls OutputDebugString
//              to display information about which driver message has been
//              passed to the DriverProc.  Special handling for the
//              DRV_EXITAPPLICATION message:  it outputs extra info to
//              notify normal-vs-abnormal exit.
//*************************************************************

void DebugDriverMessage(LPCSTR lpszMessage, LPARAM lParam1, LPARAM lParam2)
{
    if (fDebugMsgs)
    {
        wsprintf(szTmp,
            "InstDrvr: %ls\tlParam1 = %08lx, lParam2 = %08lx\n\r",
            (LPCSTR) lpszMessage,
            lParam1,
            lParam2);
        OutputDebugString(szTmp);

        if (lstrcmp(lpszMessage, "DRV_EXITAPPLICATION") == 0)
        {
            if (lParam1 == DRVEA_NORMALEXIT)
                OutputDebugString("\t\tDRVEA_NORMALEXIT\n\r");
            if (lParam1 == DRVEA_ABNORMALEXIT)
                OutputDebugString("\t\tDRVEA_ABNORMALEXIT\n\r");
        }
    }
}



//************************************************************************
//  BOOL CALLBACK _export ConfigureDialog
//
//  Description:
//      Configuration dialog displayed in DRV_CONFIGURE.  Stores info
//      in SYSTEM.INI so driver remembers its state between sessions.
//      This is the code that sets the fDebugMsgs flag in the SYSTEM.INI
//      file to control the output of debugging messages (see the
//      DebugDriverMessage function for use of this variable, and 
//      DRV_LOAD message for retrieval of setting).  It also sets up SYSTEM.INI
//      so the driver will load as Windows boots (if specified).  
//
//  Returns:
//      TRUE/FALSE per dialog box conventions.
//************************************************************************

BOOL CALLBACK _export ConfigureDialog(HWND hDlg,
                                      UINT message, 
                                      WPARAM wParam,
                                      LPARAM lParam)
{ 
    // boolean flags to indicate whether driver is being loaded at boot time
    static UINT fLoadAtBoot = 0;   // current setting
    static UINT fOldLoadAtBoot;    // previous setting

    switch (message)
    {       
    case WM_INITDIALOG:
        // Find out our current settings and check dialog buttons accordingly.
        // We already retrieved DebugMsgs when we loaded, so we only need to 
        // retrieve the LoadAtBoot setting.
        CheckDlgButton( hDlg, IDC_DEBUGMSGS, fDebugMsgs);
        CheckDlgButton( hDlg, IDC_RESTART, 0);           
        GetPrivateProfileString(szDriverName,   //section
            "LoadAtBoot",                       // entry
            "0",                                // default
            szTmp,                              // dest
            TMP_SIZE,                           // count
            "system.ini");                      // file
        CheckDlgButton(hDlg, IDC_LOADATBOOT, (szTmp[0] != '0'));
        return 1;
        
    case WM_COMMAND:
        switch (wParam)
        {       
        case IDOK:   
            // The user has selected the "Ok" button, so update settings...
            fDebugMsgs = IsDlgButtonChecked(hDlg, IDC_DEBUGMSGS);
            WritePrivateProfileString(szDriverName,     //section
                "DebugMsgs",                            // entry
                fDebugMsgs ? "1" : "0",                 // setting
                "system.ini");                          // file
            
            // Determine if we should load at boot
            GetPrivateProfileString(szDriverName,   //section
                "LoadAtBoot",                       // entry
                "0",                                // default
                szTmp,                              // dest
                TMP_SIZE,                           // count
                "system.ini");                      // file
            // see if LoadAtBoot setting has changed...
            fOldLoadAtBoot = (szTmp[0] != '0');
            fLoadAtBoot = IsDlgButtonChecked(hDlg, IDC_LOADATBOOT);
            if (fLoadAtBoot != fOldLoadAtBoot)
            {
                // if so, write the new setting to our profile string
                WritePrivateProfileString(szDriverName,     //section
                    "LoadAtBoot",                           // entry
                    (fLoadAtBoot ? "1" : "0"),              // setting
                    "system.ini");                          // file
                // and update the "drivers=" line in the [boot] section...
                if (fLoadAtBoot)
                    UpdateBootDrivers(TRUE);
                else
                    UpdateBootDrivers(FALSE);
            }
            // End the dialog and --if button is checked-- prompt to reboot...
            EndDialog(hDlg, 
                (IsDlgButtonChecked(hDlg, IDC_RESTART) 
                    ? DRVCNF_RESTART 
                    : DRVCNF_OK));
            return TRUE;
        
        case IDCANCEL:
            EndDialog(hDlg, DRVCNF_CANCEL);
            return TRUE;
        }
    }    
    return FALSE;        
} // end of ConfigureDialog()


//************************************************************************
//  BOOL UpdateBootDrivers
//
//  Description:
//      A helper routine to add or delete InstDrvr from the "drivers=" 
//      line in the [boot] section of SYSTEM.INI.  The boolean parameter
//      fAddDriver is set to TRUE if the function should add the driver to
//      the existing set of drivers; FALSE to remove it from the list of
//      drivers.  The function not only updates the "drivers=" line, but 
//      also the "LoadAtBoot=" line in the [InstDrvr] section of SYSTEM.INI.
//      Error-checking prevents it from adding it twice or attempint to 
//      remove it if it's not there.   
//************************************************************************

BOOL UpdateBootDrivers(BOOL fAddDriver)
{
    static char _far * lpDriverSubString;   // extra pointer for string search

    // retrieve current "drivers=" line from [boot] section...
    GetPrivateProfileString("boot",
        "drivers",
        "",
        szTmp,      // temporary buffer for string manipulation
        TMP_SIZE,
        "system.ini");   

    // Search for our driver name in list of drivers
    // SUGGESTED IMPROVEMENT: make the substring search case-insensitive...
    lpDriverSubString = _fstrstr(szTmp, szDriverFileName);  

    if (fAddDriver)
    {
        if (lpDriverSubString == NULL)  // only add it if it's not already
        {                               // in the string
            lstrcat(szTmp, " ");
            lstrcat(szTmp, szDriverFileName);
        }
    }    
    else // if not adding, must be removing
    {   // only remove it if it's actually in the string
        if (lpDriverSubString != NULL)  
            // The way we remove the string is to copy the tail
            // of the string (after the end of the driver name)
            // to the location of the beginning of the driver name.
            // The tail of the string begins at lpDriverSubString
            // plus the length of the driver string.
            _fstrcpy(lpDriverSubString, 
                lpDriverSubString + _fstrlen(szDriverFileName));
    }
    
    // Finally, update the "drivers=" line with new string
    WritePrivateProfileString("boot",
        "drivers",
        szTmp,
        "system.ini");
    
    return TRUE;    // success
} // end of UpdateBootDrivers() 
 
//************************************************************************
//  int CALLBACK LibMain
//
//  Description:
//      Just yer standard LibMain.
//************************************************************************

int CALLBACK LibMain(hInstance, wDataSeg, cbHeapSize, lpszNothing)
HINSTANCE   hInstance;
WORD        wDataSeg;
WORD        cbHeapSize;
LPSTR       lpszNothing;      
{   
    // Here's where library initialization code normally goes.
    // In the case of an installable driver, you can
    // use the DRV_LOAD message instead.  (LibMain will be 
    // called first, however).
    return 1;
} // end of LibMain()



//************************************************************************
//  int _far _pascal _WEP
//
//  Description:
//      This is the MS C 7 (and higher) version of a WEP.  The CRT has
//      WEP, which calls this _WEP.  If debugging messages are enabled,
//      outputs notification that _WEP has been called.
//
//  Returns:
//      1 (always)
//************************************************************************

int _far _pascal _WEP(int Param)
{   
    if (fDebugMsgs)
        OutputDebugString("InstDrvr: _WEP\n\r");
    return 1;
} // end of _WEP()
        
