/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* DESCRIPTION
*   Essential external definitions for the class Modem.
*
* CONTENTS
*   Boolean Modem::Attention  ()
*   void    Modem::ClearWait  ()
*   Boolean Modem::Dial       (char *, int, int, int)
*   Boolean Modem::Hangup     ()
*   Boolean Modem::Init       ()
*           Modem::Modem      (Modem::Comport)
*           Modem::Modem      (Modem::ComPort, unsigned, unsigned)
*           Modem::~Modem     ()
*   void    Modem::SetStdWait ()
*   void    Modem::SetWait    (int, char *)
*   int     Modem::Wait       ()
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* GLOBAL HEADER FILES
*
*****************************************************************************/

#include "modem.h"  // class Modem
#include <time.h>   // Time delays
#include <mem.h>    // Data movement

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Attention
*
* PURPOSE
*   Get the modem's attention.
*
* DESCRIPTION
*   This function attempts to get the modem's attention (ie. ready for
*   the next command).
*
*   The modem should have VERBOSE TEXT MESSAGES enabled, and EXTENDED
*   CONNECTION messages enabled (on modems that support them).
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean              // RETURN: True  = Attention succeeded
                     //         False = Attention failed

Modem::Attention()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Default return value of failure
  Boolean retval = False;

  // Purge all queues
  PurgeRx();
  PurgeTx();

  // Make 3 attempts to get modem's attention
  for( int i = 1; i <= 3 && !retval; i++ )
  {
    // Send the attention command string
    *this << commandPrefix << commandSuffix;

    // Wait 1 second for acknowledgement
    retval = ( Wait( 1 ) == MSG_OK ) ? True : False;
  }

  // Tidy up by purging the queues
  PurgeRx();
  PurgeTx();

  // Return success or failure
  return retval;
}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::ClearWait
*
* PURPOSE
*   Reset the string matching table.
*
* DESCRIPTION
*   The string matching table is reset, effecitvely making it empty. Note
*   that the table DOES NOT OWN matching strings, and so does not deallocate
*   them.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void

Modem::ClearWait()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Current table entry index
  int entry;

  // Set all entries to NULL
  for( entry = 0; entry <= 15; entry++ )
  {
    waitTable[ entry ] = NULL;
  }
}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Dial
*
* PURPOSE
*   Dial a telephone number through a modem.
*
* DESCRIPTION
*   In order to use this function, ALL the Hayes modem command info MUST be
*   setup by calls to:
*             SetCmdPrefix    SetCmdSuffix   SetDialCmd
*             SetEscapeChar   SetHangupCmd   SetInitCmd
*
*   The modem should have VERBOSE TEXT MESSAGES enabled, and EXTENDED
*   CONNECTION messages enabled (on modems that support them).
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean          // RETURN: True  = Dial succeeded - connection made
                 //         False = Dial Failed - no connection

Modem::Dial
(
  char *number,  // INPUT: Telephone number string
  int  tries,    // INPUT: Number of times to redial
  int  delay ,   // INPUT: Seconds to wait for a connection
  int  pause     // INPUT: Seconds to wait between redials
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set default return status to failure
  Boolean retval = False;

  // Sve the stream flags, and disable whitespace skipping
  long oldFlags = unsetf( skipws );

  // Temporary storage for return codes 
  int result;

  // Set standard wait messages
  SetStdWait();

  // Repeat until a connection is made, or we've redialed enough
  while( !retval && tries-- )
  {
    // Initialize connect and carrier rates
    connectRate = carrierRate = GetBitRate();

    // Get the modem's attention
    Attention();

    // Dial the number
    *this << commandPrefix << dialCommand << number << commandSuffix;

    // Wait for a message from the modem
    result = Wait( delay );

    // Check for a carrier message
    if(result == MSG_CARRIER)
    {
      // Read the carrier rate
      setf( skipws );
      *this >> carrierRate;
      unsetf( skipws );

      // Wait again for connect message
      result = Wait();
    }

    // Check for a connect message
    if( result == MSG_CONNECT )
    {
      // Read the connect rate
      if( GetBitRate() > 300UL )
      {
        setf( skipws );
        *this >> connectRate;
        unsetf( skipws );
      }

      // Dialing was successful
      retval = True;

      // Adjust carrier rate if their wasn't a message
      carrierRate = ( carrierRate ) ? carrierRate : connectRate;
    }
    else
    {
      // No connect, so hangup and pause before trying again
      Hangup();
      sleep( pause );
    }
  }
  // Restore flags
  setf( oldFlags, ~0L );

  // Return if succeeded or failed
  return retval;
}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Hangup
*
* PURPOSE
*   Command a modem too go on-hook (hangup).
*
* DESCRIPTION
*   In order to use this function, ALL the Hayes modem command info MUST be
*   setup by calls to:
*             SetCmdPrefix    SetCmdSuffix   SetDialCmd
*             SetEscapeChar   SetHangupCmd   SetInitCmd
*
*   The modem should have VERBOSE TEXT MESSAGES enabled, and EXTENDED
*   CONNECTION messages enabled (on modems that support them).
*
*   The function determines if it fails or succeeds on the presence of a
*   carrier (DCD) after trying to hangup.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean             // RETURN: True  = Hangup succeeded - no carrier
                    //         False = Hangup Failed - carrier detected

Modem::Hangup()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return value
  Boolean retval;

  // Set standard wait table strings
  SetStdWait();

  // Purge both queues
  PurgeRx();
  PurgeTx();

  // Drop DTR to cause most modems to hang up
  SetDTR( False );
  delay( 200 );
  SetDTR( True );

  // Check for a successful hangup
  if( NoCarrier() )
  {
    // Get the modem ready for the next command
    retval = Attention();
  }
  else
  {
    // Purge the queues
    PurgeRx();
    PurgeTx();

    // Go into command mode
    sleep( 2 );
    *this << escapeChar << escapeChar << escapeChar;
    sleep( 2 );

    // Command modem to hangup
    *this << commandPrefix << hangupCommand << commandSuffix;

    // Wait for modem to respond
    retval = ( Wait( 5 ) == MSG_OK ) ? True : False;
  }

  // Purge the queues
  PurgeRx();
  PurgeTx();

  // Return success or failure
  return retval;
}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Init
*
* PURPOSE
*   Command a modem to initialize itself.
*
* DESCRIPTION
*   In order to use this function, ALL the Hayes modem command info MUST be
*   setup by calls to:
*             SetCmdPrefix    SetCmdSuffix   SetDialCmd
*             SetEscapeChar   SetHangupCmd   SetInitCmd
*
*   The modem should have VERBOSE TEXT MESSAGES enabled, and EXTENDED
*   CONNECTION messages enabled (on modems that support them).
*
*   The function determines if it fails or succeeds on the presence of an
*   "OK" message after trying to initialize the modem.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Boolean             // RETURN: True  = Initialization succeeded
                    //         False = Hangup Failed

Modem::Init()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return value
  Boolean retval;

  // Set standard wait strings
  SetStdWait();

  // Try to get modem's attention
  if( ( retval = Attention() ) == True )
  {
    // Transmit the initialize command string
    *this << commandPrefix << initCommand << commandSuffix;

    // Wait for modem to respond
    retval = ( Wait( 5 ) == MSG_OK ) ? True : False;

    // Purge the queues
    PurgeRx();
    PurgeTx();
  }

  return retval;
}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Modem
*
* PURPOSE
*   Constructor.
*
* DESCRIPTION
*   This constructor does not enable a port for use. Before using the port
*   it MUST be enabled with a call to Enable().
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Modem::Modem
(
  Modem::ComPort comPort  // INPUT: COM port to use
)

/****************************************************************************
*
* BASE CLASS AND CONSTANT INITIALIZATIONS
*
*****************************************************************************/

: SerialStream( comPort )

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Reset the wait string table
  ClearWait();
}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Modem
*
* PURPOSE
*   Constructor.
*
* DESCRIPTION
*   This constructor enabls a port for use. The size of the receive and
*   transmit queues must be specified (256-65520 bytes each).
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Modem::Modem
(
  Modem::ComPort comPort,  // INPUT: COM port to use
  unsigned rxQSize,        // INPUT: Size of receive queue (256-65520 bytes)
  unsigned txQSize         // INPUT: Size of transmit queue (256-65520 bytes)
)

/****************************************************************************
*
* BASE CLASS AND CONSTANT INITIALIZATIONS
*
*****************************************************************************/

: SerialStream( comPort, rxQSize, txQSize )

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Reset the wait string table
  ClearWait();
}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::~Modem
*
* PURPOSE
*   Destructor.
*
* DESCRIPTION
*   This destructor attempts to hang up the line.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

Modem::~Modem()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Hangup on destruction
  if( IsEnabled() && !NoCarrier() )
  {
    Hangup();
  }
}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::SetStdWait
*
* PURPOSE
*   Set the wait table with standard Hayes command response codes.
*
* DESCRIPTION
*   The standard VERBOSE TEXT response codes for a Hayes modem are defined
*   in MODEM.H so that they can be easily recognized.
*
*   See Modem::Wait for details on usage.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void               // RETURN: Nothing is returned

Modem::SetStdWait()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Reset wait string table
  ClearWait();

  // Set error responses
  SetWait( MSG_ERROR,       "ERROR"       );
  SetWait( MSG_NO_DIALTONE, "NO DIALTONE" );

  // Set warning responses
  SetWait( MSG_BUSY,        "BUSY"        );
  SetWait( MSG_NO_ANSWER,   "NO ANSWER"   );
  SetWait( MSG_NO_CARRIER,  "NO CARRIER"  );
  SetWait( MSG_RING,        "RING"        );

  // Set informative responses
  SetWait( MSG_OK,          "OK"          );
  SetWait( MSG_CARRIER,     "CARRIER"     );
  SetWait( MSG_CONNECT,     "CONNECT"     );

}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::SetWait
*
* PURPOSE
*   Set a string in the wait table.
*
* DESCRIPTION
*   Up to 16 strings can be set in the wait table. Setting a string will
*   overwrite the previous one. A NULL pointer is ignored, a null string
*   is scanned but ignored.
*
*   This function expects a pointer to a string. The string is NOT OWNED by
*   the function or class, so it is up to the programmer to make sure it
*   resides in a readily accessible section of memory. Also, the string will
*   never be modifed or freed by the function or class.
*
*   See Modem::Wait for details on usage.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void               // RETURN: Nothing is returned

Modem::SetWait
(
  int entry,       // INPUT: Wait table string entry to set (1-16)
  char *string     // INPUT: A pointer to the wait table string
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set string in wait table if index is valid
  if( entry >= 1 && entry <= 16 )
  {
    waitTable[ entry - 1 ] = string;
  }
}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Wait
*
* PURPOSE
*   Wait until a string specified by SetWait() has been received, or the
*   specified number of seconds has elapsed.
*
* DESCRIPTION
*   The first and shortest string set by SetWait() in the wait string table
*   is the string that will be matched.
*
*   Example 1:
*     ClearWait();
*     SetWait(1, "Hi");
*     SetWait(2, "Hi there");
*     Wait();
*   Given the input "Hi there", string 1 will be matched. In fact, string 2
*   will NEVER be matched!
*
*   Example 2:
*     ClearWait();
*     SetWait(1, "Hi there people");
*     SetWait(2, "Hi there");
*     Wait();
*   Given input string "Hi there friends", string 2 will be matched. This is
*   correct, but 2 non-matched characters " f" were removed from the
*   serial input queue!
*
*   The rules are simple: NEVER use a string that is a prefix of another, and
*   use the shortest possible match string.
*
*   Example 3:
*     ClearWait();
*     SetWait(1, "Hi");
*     SetWait(2, " there");
*     SetWait(3, " friends");
*     SetWait(4, " people");
*   This will match all of the strings used in examples 1 & 2, and also make
*   it possible for you to tell which one was matched.
*
*   Wait() returns the index of the matched string, or 0 for a timeout.
*
*   Using a call of Wait(65535) gives the longest delay possible of over
*   17 hours (essentially forever).
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

int                 // RETURN: The index of the matched string in the
                    //         wait table, or zero for a timeout

Modem::Wait
(
  unsigned timeout  // INPUT: Number of seconds to wait for a match
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  char data[80];               // Input data buffer
  char _ss *dataPtr = data;    // Insertion point for next input character
  char _ss *dataCmpPtr = data; // Next input character to compare

  int entry = 0;                         // Wait table string entry to match
  char *tableCmpPtr = waitTable[entry];  // Next string character to compare

  Boolean noMatch = True;  // String matched flag

  clock_t stop = clock() + ( timeout * 18 ) + ( timeout / 5 );

  // Repeat until a complete string match is made
  while( noMatch )
  {
    // If there is nothing new in the comparison input buffer...
    if( dataCmpPtr == dataPtr )
    {
      while( RxIsEmpty() && stop != clock() )
      {
        SetRxFlowOn( True );
      }
      // Then input a character from the serial port
      if( RxIsEmpty() )
      {
        return 0;
      }
      else
      {
        *( dataPtr++ ) = In();
      }
    }
    // If a character match is made...
    if( ( tableCmpPtr != NULL ) && ( *dataCmpPtr == *tableCmpPtr ) )
    {
      // If the end of the wait table string has been reached...
      if( *( ++tableCmpPtr ) == '\0')
      {
        // The string has been matched exactly
        noMatch = False;

      }
      else
      {
        // Get next input character to compare
        dataCmpPtr++;
      }
    }
    else
    {
      // No match, so try next string in table
      entry = ( entry + 1 ) & 15;

      // If all strings in table have been tried...
      if( entry == 0 )
      {
        // Remove the first character from the input
        if( dataPtr > data + 1 )
        {
          memmove( data, data + 1, dataPtr - data - 1 );
        }
        // There is one less character in the input buffer now
        dataPtr--;
      }
      // Restart the comparison from the first character
      tableCmpPtr = waitTable[ entry ];
      dataCmpPtr = data;
    }
  }

  return ( entry + 1 );
}

/****************************************************************************/


