/****************************************************************************
*
* FILENAME
*   MODEM.H
*
* CLASSNAME
*   Modem
*
* PURPOSE
*   High-level stream class for modems and other serial devices.
*
* DESCRIPTION
*   This class was specifically designed to be used with Hayes type serial
*   modems, but it is not limited to only modems; any serial device can
*   enjoy the features offered by this class.
*
* INHERITANCE
*   public:
*   class SerialStream
*
* MEMBER FUNCTIONS
*   public:
*   unsigned long CarrierRate   ()
*   void          ClearWait     ()
*   unsigned long ConnectRate   ()
*   Boolean       Dial          (char *, int, int, int)
*   Boolean       Hangup        ()
*   Boolean       Init          ()
*                 Modem         (Modem &)
*                 Modem         (Modem::ComPort)
*                 Modem         (Modem::ComPort, unsigned, unsigned)
*                 ~Modem        ()
*   void          SetCmdPrefix  (char *)
*   void          SetCmdSuffix  (char *)
*   void          SetDialCmd    (char *)
*   void          SetEscapeChar (char)
*   void          SetHangupCmd  (char *)
*   void          SetInitCmd    (char *)
*   void          SetStdWait    ()
*   void          SetWait       (int, char *)
*   int           Wait          (unsigned)
*
* protected:
*   Boolean      Attention      ()
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PREPROCESSOR EXCLUSION START
*
*****************************************************************************/

#ifndef __cplusplus
#error Must use C++ for the type Modem.
#endif

#ifndef __LARGE__
#error Must use large memory model for the type Modem.
#endif

#ifndef __MODEM.H
#define __MODEM.H

/****************************************************************************
*
* GLOBAL HEADER FILES
*
*****************************************************************************/

#include "sstream.h"  // class SerialStream

/***************************************************************************
*
* CLASS DEFINITION START
*
*****************************************************************************/

class Modem
: public SerialStream
{
  /****************************************/

  // String matching table

  private:
  char *waitTable[ 16 ];  // Maximum of 16 string entries

  /****************************************/

  // Hayes Type Modem Command Strings

  private:
  char *initCommand;    // Initialization Command
  char *dialCommand;    // Dial Command
  char *hangupCommand;  // Hangup Command
  char *commandPrefix;  // Command Prefix
  char *commandSuffix;  // Command Suffix
  char escapeChar;      // Command Mode Escape Character

  /****************************************/

  // Data Rates

  private:
  unsigned long connectRate;  // Connection Data Rate (DTE-DCE)
  unsigned long carrierRate;  // Carrier Data Rate (DCE-DCE)

  /****************************************/

  // Default Command Response Messages

  public:
  enum {
    MSG_ERROR       = 1,  // "ERROR"
    MSG_NO_DIALTONE = 2,  // "NO DIALTONE"
    MSG_FATAL       = 2,  // The above are usually fatal errors

    MSG_BUSY        = 3,  // "BUSY"
    MSG_NO_ANSWER   = 4,  // "NO ANSWER"
    MSG_NO_CARRIER  = 5,  // "NO CARRIER"
    MSG_RING        = 6,  // "RING"
    MSG_WARNING     = 6,  // The above are warning messages

    MSG_OK          = 7,  // "OK"
    MSG_CARRIER     = 8,  // "CARRIER"
    MSG_CONNECT     = 9,  // "CONNECT"
    MSG_INFO        = 9   // The above are informational messages
  };

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Attention
*
* PURPOSE
*   Get the modem's attention.
*
* DESCRIPTION
*   This function attempts to get the modem's attention (ie. ready for
*   the next command).
*
*   The modem should have VERBOSE TEXT MESSAGES enabled, and EXTENDED
*   CONNECTION messages enabled (on modems that support them).
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected:

Boolean              // RETURN: True  = Attention succeeded
                     //         False = Attention failed

Modem::Attention();

/****************************************************************************
*
* FILENAME
*   MODEM.H
*
* FUNCTION NAME
*  Modem::CarrierRate
*
* PURPOSE
*   Get the current carrier bit rate (bits per second).
*
* DESCRIPTION
*   The carrier rate reflects the data bit rate in use between two modems
*   (DCE-DCE). The rate is detected ONLY by Dial(), and ONLY if extended
*   messages are enabled on the modem. The current UART bit rate is
*   returned otherwise.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

unsigned long   // RETURN: The carrier bit rate in use

Modem::CarrierRate()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return the current carrier bit rate
  return ( carrierRate ) ? carrierRate : GetBitRate();
}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::ClearWait
*
* PURPOSE
*   Reset the string matching table.
*
* DESCRIPTION
*   The string matching table is reset, effecitvely making it empty. Note
*   that the table DOES NOT OWN matching strings, and so does not deallocate
*   them.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

void

Modem::ClearWait();

/****************************************************************************
*
* FILENAME
*   MODEM.H
*
* FUNCTION NAME
*  Modem::ConnectRate
*
* PURPOSE
*   Get the current connection bit rate (bits per second).
*
* DESCRIPTION
*   The carrier rate reflects the data bit rate in use between the modem
*   and the terminal (DTE-DCE). The rate is detected ONLY by Dial(), and
*   ONLY if connection messages are enabled on the modem. The current UART
*   bit rate is returned otherwise.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

unsigned long   // RETURN: The connection bit rate in use

Modem::ConnectRate()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return the current connection bit rate
  return ( connectRate ) ? connectRate : GetBitRate();
}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Dial
*
* PURPOSE
*   Dial a telephone number through a modem.
*
* DESCRIPTION
*   In order to use this function, ALL the Hayes modem command info MUST be
*   setup by calls to:
*             SetCmdPrefix    SetCmdSuffix   SetDialCmd
*             SetEscapeChar   SetHangupCmd   SetInitCmd
*
*   The modem should have VERBOSE TEXT MESSAGES enabled, and EXTENDED
*   CONNECTION messages enabled (on modems that support them).
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Boolean             // RETURN: True  = Dial succeeded - connection made
                    //         False = Dial Failed - no connection

Modem::Dial
(
  char *number,     // INPUT: Telephone number string
  int  tries = 1,   // INPUT: Number of times to redial
  int  delay = 30,  // INPUT: Seconds to wait for a connection
  int  pause = 4    // INPUT: Seconds to wait between redials
);

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Hangup
*
* PURPOSE
*   Command a modem too go on-hook (hangup).
*
* DESCRIPTION
*   In order to use this function, ALL the Hayes modem command info MUST be
*   setup by calls to:
*             SetCmdPrefix    SetCmdSuffix   SetDialCmd
*             SetEscapeChar   SetHangupCmd   SetInitCmd
*
*   The modem should have VERBOSE TEXT MESSAGES enabled, and EXTENDED
*   CONNECTION messages enabled (on modems that support them).
*
*   The function determines if it fails or succeeds on the presence of a
*   carrier (DCD) after trying to hangup.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Boolean             // RETURN: True  = Hangup succeeded - no carrier
                    //         False = Hangup Failed - carrier detected

Modem::Hangup();

/****************************************************************************
*
* FILENAME
*   MODEM.H
*
* FUNCTION NAME
*  Modem::Init
*
* PURPOSE
*   Command a modem to initialize itself.
*
* DESCRIPTION
*   In order to use this function, ALL the Hayes modem command info MUST be
*   setup by calls to:
*             SetCmdPrefix    SetCmdSuffix   SetDialCmd
*             SetEscapeChar   SetHangupCmd   SetInitCmd
*
*   The modem should have VERBOSE TEXT MESSAGES enabled, and EXTENDED
*   CONNECTION messages enabled (on modems that support them).
*
*   The function determines if it fails or succeeds on the presence of an
*   "OK" message after trying to initialize the modem.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Boolean             // RETURN: True  = Initialization succeeded
                    //         False = Hangup Failed

Modem::Init();

/****************************************************************************
*
* FILENAME
*   MODEM.H
*
* FUNCTION NAME
*  Modem::Modem
*
* PURPOSE
*   Copy constructor.
*
* DESCRIPTION
*   This function is declared but not defined to prevent copying.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Modem::Modem
(
  Modem &
);

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Modem
*
* PURPOSE
*   Constructor.
*
* DESCRIPTION
*   This constructor does not enable a port for use. Before using the port
*   it MUST be enabled with a call to Enable().
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Modem::Modem
(
  Modem::ComPort comPort // INPUT: COM port to use
);

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Modem
*
* PURPOSE
*   Constructor.
*
* DESCRIPTION
*   This constructor enabls a port for use. The size of the receive and
*   transmit queues must be specified (256-65520 bytes each).
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Modem::Modem
(
  Modem::ComPort comPort,  // INPUT: COM port to use
  unsigned rxQSize,        // INPUT: Size of receive queue (256-65520 bytes)
  unsigned txQSize         // INPUT: Size of transmit queue (256-65520 bytes)
);

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::~Modem
*
* PURPOSE
*   Destructor.
*
* DESCRIPTION
*   This destructor attempts to hang up the line.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: virtual

Modem::~Modem();

/****************************************************************************
*
* FILENAME
*   MODEM.H
*
* FUNCTION NAME
*  Modem::SetCmdPrefix
*
* PURPOSE
*   Set the command prefix string.
*
* DESCRIPTION
*   Normally the Hayes command prefix is "AT". This function expects a
*   pointer to such a string. The string is NOT OWNED by the function
*   or class, so it is up to the programmer to make sure it resides
*   in a readily accessible section of memory. Also, the string will never
*   be modifed or freed by the function or class.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void            // RETURN: Nothing is returned

Modem::SetCmdPrefix
(
  char *cmd     // INPUT: A pointer to the command prefix string
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the command prefix string
  commandPrefix = cmd;
}

/****************************************************************************
*
* FILENAME
*   MODEM.H
*
* FUNCTION NAME
*  Modem::SetCmdSuffix
*
* PURPOSE
*   Set the command suffix string.
*
* DESCRIPTION
*   Normally the Hayes command suffix is an ASCII carriage return. This
*   function expects a pointer to such a string. The string is NOT OWNED by
*   the function or class, so it is up to the programmer to make sure it
*   resides in a readily accessible section of memory. Also, the string will
*   never be modifed or freed by the function or class.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void            // RETURN: Nothing is returned

Modem::SetCmdSuffix
(
  char *cmd     // INPUT: A pointer to the command suffix string
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the command suffix string
  commandSuffix = cmd;
}

/****************************************************************************
*
* FILENAME
*   MODEM.H
*
* FUNCTION NAME
*  Modem::SetDialCmd
*
* PURPOSE
*   Set the dial command string.
*
* DESCRIPTION
*   Normally the Hayes dial command prefix is "DT" for tone dialing, or "DP"
*   for pulse dialing. This function expects a pointer to such a string. The
*   string is NOT OWNED by the function or class, so it is up to the
*   programmer to make sure it resides in a readily accessible section of
*   memory. Also, the string will never be modifed or freed by the function
*   or class.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void            // RETURN: Nothing is returned

Modem::SetDialCmd
(
  char *cmd     // INPUT: A pointer to the dial command string
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the dial command string
  dialCommand = cmd;
}

/****************************************************************************
*
* FILENAME
*   MODEM.H
*
* FUNCTION NAME
*  Modem::SetEscapeChar
*
* PURPOSE
*   Set the escape to command mode character.
*
* DESCRIPTION
*   Normally the Hayes escape character is "+".
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void            // RETURN: Nothing is returned

Modem::SetEscapeChar
(
  char cmd     // INPUT: The command mode escape character
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the command mode escape charater
  escapeChar = cmd;
}

/****************************************************************************
*
* FILENAME
*   MODEM.H
*
* FUNCTION NAME
*  Modem::SetHangupCmd
*
* PURPOSE
*   Set the hangup command string.
*
* DESCRIPTION
*   Normally the Hayes hangup command is "H". This function expects a
*   pointer to such a string. The string is NOT OWNED by the function
*   or class, so it is up to the programmer to make sure it resides
*   in a readily accessible section of memory. Also, the string will never
*   be modifed or freed by the function or class.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void            // RETURN: Nothing is returned

Modem::SetHangupCmd
(
  char *cmd     // INPUT: A pointer to the hangup command string
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the hangup command string
  hangupCommand = cmd;
}

/****************************************************************************
*
* FILENAME
*   MODEM.H
*
* FUNCTION NAME
*  Modem::SetInitCmd
*
* PURPOSE
*   Set the initialization command string.
*
* DESCRIPTION
*   Normally the Hayes initialization command is "Z", but it can be quite
*   complex to set up specific features. This function expects a pointer to
*   such a string. The string is NOT OWNED by the function or class, so it
*   is up to the programmer to make sure it resides in a readily accessible
*   section of memory. Also, the string will never be modifed or freed by
*   the function or class.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void               // RETURN: Nothing is returned

Modem::SetInitCmd
(
  char *cmd        // INPUT: A pointer to the initialization command string
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the initialization command string
  initCommand = cmd;
}

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::SetStdWait
*
* PURPOSE
*   Set the wait table with standard Hayes command response codes.
*
* DESCRIPTION
*   The standard VERBOSE TEXT response codes for a Hayes modem are defined
*   in MODEM.H so that they can be easily recognized.
*
*   See Modem::Wait for details on usage.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void               // RETURN: Nothing is returned

Modem::SetStdWait();

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::SetWait
*
* PURPOSE
*   Set a string in the wait table.
*
* DESCRIPTION
*   Up to 16 strings can be set in the wait table. Setting a string will
*   overwrite the previous one. A NULL pointer is ignored, a null string
*   is scanned but ignored.
*
*   This function expects a pointer to a string. The string is NOT OWNED by
*   the function or class, so it is up to the programmer to make sure it
*   resides in a readily accessible section of memory. Also, the string will
*   never be modifed or freed by the function or class.
*
*   See Modem::Wait for details on usage.
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

void               // RETURN: Nothing is returned

Modem::SetWait
(
  int entry,       // INPUT: Wait table string entry to set (1-16)
  char *string     // INPUT: A pointer to the wait table string
);

/****************************************************************************
*
* FILENAME
*   MODEM.CPP
*
* FUNCTION NAME
*  Modem::Wait
*
* PURPOSE
*   Wait until a string specified by SetWait() has been received, or the
*   specified number of seconds has elapsed.
*
* DESCRIPTION
*   The first and shortest string set by SetWait() in the wait string table
*   is the string that will be matched.
*
*   Example 1:
*     ClearWait();
*     SetWait(1, "Hi");
*     SetWait(2, "Hi there");
*     Wait();
*   Given the input "Hi there", string 1 will be matched. In fact, string 2
*   will NEVER be matched!
*
*   Example 2:
*     ClearWait();
*     SetWait(1, "Hi there people");
*     SetWait(2, "Hi there");
*     Wait();
*   Given input string "Hi there friends", string 2 will be matched. This is
*   correct, but 2 non-matched characters " f" were removed from the
*   serial input queue!
*
*   The rules are simple: NEVER use a string that is a prefix of another, and
*   use the shortest possible match string.
*
*   Example 3:
*     ClearWait();
*     SetWait(1, "Hi");
*     SetWait(2, " there");
*     SetWait(3, " friends");
*     SetWait(4, " people");
*   This will match all of the strings used in examples 1 & 2, and also make
*   it possible for you to tell which one was matched.
*
*   Wait() returns the index of the matched string, or 0 for a timeout.
*
*   Using a call of Wait(65535) gives the longest delay possible of over
*   17 hours (essentially forever).
*
* EXTERNAL REFERENCES
*   class Modem
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   MODEM.OBJ
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

int                      // RETURN: The index of the matched string in the
                         //         wait table, or zero for a timeout

Modem::Wait
(
  unsigned timeout = 30  // INPUT: Number of seconds to wait for a match
);

/***************************************************************************
*
* CLASS DEFINITION END
*
*****************************************************************************/

};

/****************************************************************************
*
* PREPROCESSOR EXCLUSION END
*
*****************************************************************************/

#endif
