// ------------------------------------------------------------------------
//
//   Create a class that handles multiple screen dialog boxes
//
//   Parent dialog box must contain a groupbox (or other control) which is
//   the same size and position as the area to be used for subdialogs.  This
//   groupbox (or other control) will not be included in the final dialog
//   box.  The parent dialog box must also contain a listbox to which the
//   subdialog names are added using the AddSelection() member function.
//   The data associated with each item in the listbox is set by
//   CMultiDialog and should not be altered.  Subdialog names should not be
//   duplicated in the list.  Do not call ShowWindow() on controls in
//   subdialogs.
//
//   Copyright 1993, Chris Shearer Cooper  CompuServe 76666,3026
//   This file may be copied and distributed free of charge as long as
//   this copyright notice remains intact and any changes by other
//   authors are clearly indicated.
//   This code may be used in your application without any monetary
//   royalty, however payment is required in the form of feedback and
//   bug reports to the author.  Although every effort has been made
//   to make this code bug-free, if you use this code in your application
//   the author takes no responsibility for any damages which may occur
//   from the operation of this code.
//
// ------------------------------------------------------------------------

#ifndef __MULTIDLG_H

#include "map_wl.h"
#include "template.h"

// Every time the CMultiDialog object changes which subtemplate is selected, it
// sends this message to the dialog with :
//		wParam			= # of subtemplate now selected (first subtemplate = 0)
//		LOWORD(lParam)	= Reason for change
//							MDN_USER_SETSEL = user called SetSelection()
//							MDN_UPDATEDATA_FAIL = UpdateData() failed and set
//								focus to an item in an inactive subtemplate
//		HIWORD(lParam)	= 0
#define MDN_SELECTCHANGE		(WM_USER + 0x6322)
#define MDN_USER_SETSEL			0x6800
#define MDN_UPDATEDATA_FAIL		0x6801


class CMultiDialog : public CDialog
	{
	DECLARE_DYNAMIC(CMultiDialog)
public:
	// Construct a CMultiDialog from various places
	__export CMultiDialog(LPCSTR szBaseTemplate, UINT uIDSizeBox, CWnd *pParent = NULL);
	__export CMultiDialog(UINT nIDBaseTemplate, UINT uIDSizeBox, CWnd *pParent = NULL);
	__export ~CMultiDialog(void);

	// Once the dialog box is displayed (with DoModal()) you cannot add any
	// more selections.  If szText is NULL, uses the caption from the dialog
	// box template. Will throw a resource exception if a problem occurs.
	void __export AddSelection(LPCSTR szSubTemplate, LPCSTR szText = NULL);
	void AddSelection(UINT nIDSubTemplate, LPCSTR szText = NULL)
		{ AddSelection(MAKEINTRESOURCE(nIDSubTemplate), szText); }

	int __export DoModal();

	// Call this to fill a list box with the AddSelection() item names
	BOOL __export FillListBox(CListBox *pListBox, int iSelect = -1);

	// The selection can only be set when the dialog is active (during DoModal()).  This
	// function must be called (at least) once from your OnInitDialog().
	BOOL __export SetSelection(LPCSTR szText, BOOL bEraseBkgnd = TRUE);
	// 0 = first AddSelection() item
	BOOL __export SetSelection(int iWhich, BOOL bEraseBkgnd = TRUE, int iReason = MDN_USER_SETSEL);

	// _NEVER_ call ShowWindow directly on a control in a CMultiDialog!
	BOOL __export ShowDlgItem(int nID, int nCmdShow = SW_SHOW);

	// If you create a window manually which should be made visible & invisible
	// along with a subtemplate, call this function.
	BOOL __export AddDlgItem(HWND hWnd, int iWhichSubTemplate);

	// Make sure to call this instead of CDialog's version
	BOOL __export UpdateData(BOOL bSaveAndValidate = TRUE);

	// Returns an ID which is not used in the template
	UINT UnusedID(void)
		{ return m_sTemplate.unused_id(); }

protected:
	BOOL __export init(LPCSTR szBaseTemplate, UINT uIDGroupBox, CWnd *pParent);

	BOOL __export ConstructTemplate(void);

	CDialogTemplate m_sTemplate;
	CRect m_rectSub;			// In dialog units
	CObArray m_aMaps;			// Array of mappings between text and subtemplate controls
	int m_nMainTemplateItems;
	int m_iFirstEnabledItem;
	int m_nEnabledItems;
	int m_iInsertIndex;			// Index in m_sTemplate of first control after subtemplates
	CMapWordToLong m_aStatus;	// Array of control status values
	};


#define __MULTIDLG_H
#endif


