#include "parse.h"



/*
 * parse_data: This routine takes a pointer to a data buffer containing the
 * information read off the card and the length (amount) of information.
 * The required info is then displayed, and the Tag parsing routines are
 * called.
 */
void
parse_data (BYTE *data,ULONG nBytes)
{

	ULONG   nCur=0;
	UINT	TagLen;
	BYTE 	Tag;

	//Vendor ID is first 4 bytes, in Packed Ascii.
	LogPrintf("Vendor ID:     ");
	display_comp_asc(data[nCur],data[nCur+1]);
	LogPrintf("%04X",	data[nCur+2]+(data[nCur+3] << 8));
	LogPrintf("\n");

	//nCur is not yet set
	nCur=VEND_ID_LEN;

	//The Serial number is 4 bytes, 4 bytes in from the start of the string
	//In Decending order...

	LogPrintf("Serial Number: ");
	for (nCur;nCur<DEV_CODE_OFFSET;nCur++) {
		LogPrintf("%02X",data[DEV_CODE_OFFSET+VEND_ID_LEN-nCur-1]);
	}
	LogPrintf("\n");

	LogPrintf("Checksum (reported): 0x%02X\n",data[nCur++]);



	do {
		Tag=GetTag(data,nCur,&TagLen);
		//Add one to point to the start of the next tag
		nCur+=DisplayTagInfo (Tag,data,nCur,TagLen);

	} while (Tag != PNP_END);


}


//Displays a Byte in binary format
void 
display_bin (BYTE Num) 
{

	BYTE i;

	//8 bits per byte...hopefully
	for (i=sizeof(BYTE)*8;i>0;i--) {
		if ((Num >> i-1 ) & 1) {
			LogPrintf("1");
		}
		else {
			LogPrintf("0");
		}
	}
	LogPrintf("\n");
}


//Displays two Bytes (a word) stored in Compress ascii, as Chars.
void
display_comp_asc (BYTE Hbyte,BYTE Lbyte)
{

	WORD Word;
	BYTE c;

	//Build the Word
	Word=Lbyte+(Hbyte << 8);


	//Hack o'rama. Shift over each letter, mask out the rest, then print
    //it if it is in the character space.

	c=(BYTE)(Word >> 10 & 0x1f) + 'A'-1;	
	LogPrintf("%c",(c >= 'A' && c <= 'Z')?c:'\0'); 

	c=(BYTE)(Word >> 5 & 0x1f) + 'A'-1;
	LogPrintf("%c",(c >= 'A' && c <= 'Z')?c:'\0');

	c=(BYTE)(Word & 0x1f) + 'A'-1;
	LogPrintf("%c",(c >= 'A' && c <= 'Z')?c:'\0'); 

	
}



/*
 *	The First thing we expect to see at this point is a tag. Hopefully
 *  we'll see a tag we recognize and report it. 
 *  
 *  Entry: *data -- pointer to the BEGINNING of the data buffer. 
 *         nCur  -- the current offset in the data buffer
 *		   *pLen -- pointer to a uint returning the size of the tag
 */
BYTE
GetTag(BYTE *data,ULONG nCur,UINT *pLen)
{

	BYTE name;

	if (fVerbose) {
		LogPrintf("\nExamining Byte: 0x%02X\n",data[nCur]);
	}

	//Find out if it's a small or large tag.
	if (data[nCur] >> 7 ) {

		//High bit is on, it's a long Tag so leave on the high bit.
        //So Display Tag info can discriminate
		name=data[nCur];

		//The Length of the Tag is the next two bytes
		*pLen=(data[nCur+2] << 8)+(data[nCur+1]);


		//if verbose/debug
		if (fVerbose) {
			LogPrintf("Large Tag:    0x%02X\n",name);
			LogPrintf("Length: %d\n",*pLen);
		}

	}
	else {

		//Small item....
		name=data[nCur] >> 3;
		*pLen=data[nCur] & 0x7;
		if (fVerbose) {
			LogPrintf("Tag:    0x%02X\n",name);
			LogPrintf("Length: %d\n",*pLen);
		}

	}

	return (name);

}


/*
 * DisplayTagInfo: This routine takes the following parameters:
 *	Tag : The actual tag found, shifted to the bottom of the word
 *	Data: A pointer to the data stream
 *	nCur: The current offset in the data stream
 *	Len : The Length of the tag being evaluated
 *
 *	This routine checks the tag, dispatches most cases to the display code,
 *	or processes the smaller cases itself
 *
 *	Returns: The Length proccessed.
 */
 
UINT
DisplayTagInfo (BYTE Tag,BYTE *data,ULONG nCur,UINT Len)
{

	UINT i;

	//We now have the tag. We could mix this in with the resolution
    //Loop. But it has been separated for clarity.


	//Make sure we toggle the empty logical config flag.
	if (Tag != PNP_START_DF && Tag != PNP_END_DF) {
		fEmpty=FALSE;
	}

	if (Tag >> 7) {

		//Long Tag.	Clear the high bit;
		Tag ^= 0x80;

		//Add the tag length
		Add_Large(nCur);


		switch (Tag) {

			//Memory Descriptor block
			case PNP_L_MEM_RD: {
				display_Mem_RD(data,nCur,Len);
				break;
			}

			//Ansi Device Identifier
			case PNP_L_IDSTRNGA: {
				LogPrintf("Device Description: ");

				for (i=0;i<Len;i++) {
					LogPrintf("%c",data[nCur+i]);
				}
				LogPrintf("\n");
				break;
			}

			//Unicode Device Identifier, not yet defined
			case PNP_L_IDSTRNGU: {
				LogPrintf("!ERROR! Unicode Tags are not yet defined...\n");
				break;
			}

			//Vendor Defined Tag...
			case PNP_L_VENDOR: {
				LogPrintf("Vendor Defined Data:\n");
				for (i=0;i<Len;i++) {
					LogPrintf("%02X ",data[nCur+i]);
				}
				LogPrintf("\n");
				break;

			}

			default: {
				LogPrintf("Saw a large tag, in the reserved range.\n");
				LogPrintf("Tag: 0x%02X\n\n",Tag);
				break;
			}
		}
		return (Len+LARGE_HEADER);
		
	} 

	else {
		//Small tag

		//Account for the header.
		Add_Small(nCur);	

		switch (Tag) {
			case PNP_VERSION: {
				//Display PNP version info

				if (Len != VER_LEN) {
					LogPrintf("Version Length Field of non-standard length\nDisplaying raw data\n");
					//Display length chars...
					i=Len;
					while (i) {
						i--;
						LogPrintf("%02X",data[nCur+i]);
					}
				}
				else {
					//Bits 4-7 Major, 0-3 Minor
					LogPrintf("PNP Version:   %d.%d\n",data[nCur] >> 4,data[nCur] & 0xf);
					LogPrintf("Vendor Ver.:   %02X",data[nCur+1]);
				}
				LogPrintf("\n");
				break;
			}

			
			case PNP_LOG_DEV_ID: {
				//Logical Device ID
				display_Log_Dev_ID(data,nCur,Len);
				break;
			}
			case PNP_COMP_DEV_ID: {
				//Show the compatible device ID
				LogPrintf("Compatible Device ID:");
				LogPrintf("0x%08X\n",	(data[nCur+3] << 24)+(data[nCur+2] << 16)+(data[nCur+1] << 8)+(data[nCur]));

				break;
			}
			case PNP_IRQ_FORMAT: {
				//IRQ Requirements
				display_IRQ_FMT(data,nCur,Len);
				break;
			}
			case PNP_DMA_FORMAT: {
				//DMA Format and requirements
				display_DMA_FMT(data,nCur,Len);
				break;
			}
			case PNP_START_DF: {

				if (fEmpty) {
					LogPrintf("Empty Dependant Function (No requirements)\n\n");
				}
				//Start of a Dependant Function
				LogPrintf("\nDependant Function %d\n",nStartDF++);
				if (Len) {
					if (!data[nCur]) {
						LogPrintf("Best Possible Configuration\n");
					}
					else {
						if (data[nCur] == 1) {
							LogPrintf("Acceptable Configuration\n");
						}
						else {
							if (data[nCur] == 2) {
								LogPrintf("Sub-Optimal Configuration\n");
							}
							else {
								LogPrintf("!ERROR! Reserved configuration Selection");
							}
						}
					}
				}
				else {
					LogPrintf("Acceptable Configuration\n");
				}
				fEmpty=TRUE;
				break;
			}
			case PNP_END_DF: {
				if (!nStartDF) {
					LogPrintf("!ERROR! Saw an End DF Tag with no StartDF\n");
				}
				if (fEmpty) {
					LogPrintf("Empty Dependant Function (No requirements)\n\n");
				}

				LogPrintf("End of Dependant Functions\n");

				if (fEndDF) 
					LogPrintf("!ERROR! Multiple End DF Tags\n");
				fEndDF=TRUE;
				break;
			}
			case PNP_IO_DESC: {
				// I/O Descriptor (relocatable)
				display_IO_DESC(data,nCur,Len);
				break;
			}
			case PNP_IO_FIXED: {
				// I/O Descriptor (fixed)
				display_Fixed_DESC(data,nCur,Len);
				break;
			}
			case PNP_RESERVED: 
			case PNP_RESERVED1:
			case PNP_RESERVED2:
			case PNP_RESERVED3: {
				LogPrintf("Saw a small tag in the reserved range.\n");
				LogPrintf("Tag: 0x%02X\n\n",Tag);
				break;
			}

			//Vendor Defined Tag...
			case PNP_VENDOR: {
				LogPrintf("Vendor Defined Data:\n");
				for (i=0;i<Len;i++) {
					LogPrintf("%02X ",data[nCur+i]);
				}
				LogPrintf("\n");
				break;

			}

			case PNP_END: {
				break;
			}

			default: {
		 		LogPrintf("!ERROR! Fell through Small Tags...\n\n");
				break;
			}
		}
		return (Len+SMALL_HEADER);
	}
}

