//  SAY.H
//
//  Macros to provide printf-like notification messages in WinApps.
//  Say(...)        redefined as either MBSay() or CommSay()
//  MBSay(...)      outputs to a MessageBox.
//  CommSay(...)    outputs to the debugging monitor on COM1.
//  DebugSay(...)   outputs to debugging monitor only in debug build.
//
//  Each of the above has 5 flavors, depending on how many arguments 
//  there are.  This is because macros don't have variable-length 
//  argument lists.  Using Say() as an example:
//      Say(szMsg) - outputs the string
//  The remainder work like printf() ... the format string specifies
//  how the parameters are to be converted to ascii.
//      Say1(szFormat, param) 
//      Say2(szFormat, param1, param2) 
//      Say3(szFormat, param1, param2, param3)
//      Say4(szFormat, param1, param2, param3, param4)
//
// Say() defaults to MBSay unless specified otherwise...

#if !defined(COMMSAY) && !defined(MBSAY)
#define MBSAY
#endif

#if defined(MBSAY)
    #define Say(szMessage) \
          MBSay(szMessage)
    #define Say1(format, param1) \
          MBSay1(format, param1)
    #define Say2(format, param1, param2) \
          MBSay2(format, param1, param2)
    #define Say3(format, param1, param2, param3) \
          MBSay3(format, param1, param2, param3)
    #define Say4(format, param1, param2, param3, param4) \
          MBSay4(format, param1, param2, param3, param4)
#else
#if defined(COMMSAY)
    #define Say(szMessage) \
        CommSay(szMessage)
    #define Say1(format, param1) \
        CommSay1(format, param1)
    #define Say2(format, param1, param2) \
        CommSay2(format, param1, param2)
    #define Say3(format, param1, param2, param3) \
        CommSay3(format, param1, param2, param3)
    #define Say4(format, param1, param2, param3, param4) \
        CommSay4(format, param1, param2, param3, param4)
#endif
#endif

// MBSay() accepts a single message to output in a messagebox.
#define MBSay(szMessage) \
{                       \
    char szCaption[80]; \
                        \
    wsprintf(szCaption, "Say (%s:%d)", (LPSTR) __FILE__, __LINE__); \
    MessageBox(0, szMessage, szCaption, MB_OK|MB_TASKMODAL); \
}

// MBSay1() accepts single format-specifier and one parameter to match it...
#define MBSay1(format, param) \
{ \
    char szText[160]; \
    char szCaption[80]; \
                        \
    wsprintf(szText, format, param); \
    wsprintf(szCaption, "Say (%s:%d)", (LPSTR) __FILE__, __LINE__); \
    MessageBox(0, szText, szCaption, MB_OK|MB_TASKMODAL); \
}

// MBSay2() is for a format-specifier with 2 arguments...
#define MBSay2(format, param1, param2) \
{ \
    char szText[160]; \
    char szCaption[80]; \
                        \
    wsprintf(szText, format, param1, param2); \
    wsprintf(szCaption, "Say (%s:%d)", (LPSTR) __FILE__, __LINE__); \
    MessageBox(0, szText, szCaption, MB_OK|MB_TASKMODAL); \
}

// MBSay3() is for a format-specifier with 3 arguments...
#define MBSay3(format, param1, param2, param3) \
{ \
    char szText[160]; \
    char szCaption[80]; \
                        \
    wsprintf(szText, format, param1, param2, param3); \
    wsprintf(szCaption, "Say (%s:%d)", (LPSTR) __FILE__, __LINE__); \
    MessageBox(0, szText, szCaption, MB_OK|MB_TASKMODAL); \
}

// MBSay4() is for a format-specifier with 4 arguments...
#define MBSay4(format, param1, param2, param3, param4) \
{ \
    char szText[160]; \
    char szCaption[80]; \
                        \
    wsprintf(szText, format, param1, param2, param3, param4); \
    wsprintf(szCaption, "Say (%s:%d)", (LPSTR) __FILE__, __LINE__); \
    MessageBox(0, szText, szCaption, MB_OK|MB_TASKMODAL); \
}


// The CommSay routines are just like the Say routines, but output
// goes to the debugging monitor on the comm port...

#define CommSay(szMessage) \
{ \
    OutputDebugString (szMessage); \
    OutputDebugString ("\n\r");\
}

// CommSay1 is for a format-specifier with 1 argument...

#define CommSay1(format, param) \
{ \
    char szText[160]; \
                        \
    wsprintf(szText, format, param); \
    lstrcat (szText, "\n\r"); \
    OutputDebugString (szText); \
}

// CommSay2 is for a format-specifier with 2 arguments...
#define CommSay2(format, param1, param2) \
{ \
    char szText[160]; \
                        \
    wsprintf(szText, format, param1, param2); \
    lstrcat (szText, "\n\r"); \
    OutputDebugString (szText); \
}

// CommSay3 is for a format-specifier with 3 arguments...
#define CommSay3(format, param1, param2, param3) \
{ \
    char szText[160]; \
                        \
    wsprintf(szText, format, param1, param2, param3); \
    lstrcat (szText, "\n\r"); \
    OutputDebugString (szText); \
}

// CommSay4 is for a format-specifier with 4 arguments...
#define CommSay4(format, param1, param2, param3, param4) \
{ \
    char szText[160]; \
                        \
    wsprintf(szText, format, param1, param2, param3, param4); \
    lstrcat (szText, "\n\r"); \
    OutputDebugString (szText); \
}

// The DebugSay*() macros are identical to the CommSay*() macros except
// that no code is generated unless in DEBUG build...

#ifdef DEBUG    // generate code only if this is a debug build...

#define DebugSay(szMessage)                 CommSay(szMessage)
#define DebugSay1(format, param)            CommSay1(format, param)
#define DebugSay2(format, p1, p2)           CommSay2(format, p1, p2)
#define DebugSay3(format, p1, p2, p3)       CommSay2(format, p1, p2, p3)
#define DebugSay4(format, p1, p2, p3, p4)   CommSay2(format, p1, p2, p3, p4)

#else  // don't generate any code for retail build...

#define DebugSay(szMessage)  
#define DebugSay1(format, param)    
#define DebugSay2(format, p1, p2)    
#define DebugSay3(format, p1, p2, p3)    
#define DebugSay4(format, p1, p2, p3, p4)    

#endif
