/****************************************************************************
*
* FILENAME
*   SERIAL.H
*
* ClASSNAME
*   Serial
*
* PURPOSE
*    An interrupt driven byte oriented serial class for COM1-COM4.
*
* DESCRIPTION
*    This class features the following:
*      - interrupt driven receiver and transmitter
*      - interrupt sharing
*      - on the fly adjustable transmit and receive queue sizes
*      - hardware (RTS/CTS) or software (XON/XOFF) capable
*      - automatic disk access flow control
*      - universal serial port usage that is not restricted to modems
*
* INHERITANCE
*   public:
*   class Uart
*
* MEMBER FUNCTIONS
*   public:
*   Boolean       Serial::AllSent       ()
*   Boolean       Serial::AnyErrors     ()
*   void          Serial::Break         (unsigned)
*   Boolean       Serial::BreakDetect   ()
*   void          Serial::ClearErrors   ()
*   void          Serial::Disable       ()
*   Boolean       Serial::Enable        (unsigned, unsigned)
*   Boolean       Serial::FrameError    ()
*   unsigned      Serial::GetRxCount    ()
*   unsigned      Serial::GetTxCount    ()
*   unsigned char Serial::In            ()
*   Boolean       Serial::IsEnabled     ()
*   Boolean       Serial::NoCarrier     ()
*   void          Serial::Out           (unsigned char)
*   Boolean       Serial::OverflowError ()
*   Boolean       Serial::ParityError   ()
*   unsigned char Serial::PeekRx        (unsigned)
*   void          Serial::PurgeRx       ()
*   void          Serial::PurgeTx       ()
*   Boolean       Serial::RxIsFull      ()
*   Boolean       Serial::RxIsEmpty     ()
*                 Serial::Serial        (Serial &)
*                 Serial::Serial        (Serial::ComPort)
*                 Serial::Serial        (Serial::ComPort, unsigned, unsigned)
*                 Serial::~Serial       ()
*   void          Serial::SetComPort    (Serial::ComPort)
*   void          Serial::SetFlowType   (Serial::FlowType)
*   void          Serial::SetRxFlowOn   (Boolean)
*   void          Serial::SetTxFlowOn   (Boolean)
*   Boolean       Serial::TxIsFull      ()
*   Boolean       Serial::TxIsEmpty     ()
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PREPROCESSOR EXCLUSION START
*
*****************************************************************************/

#ifndef __cplusplus
#error Must use C++ for the type Serial.
#endif

#ifndef __LARGE__
#error Must use large memory model for the type Serial.
#endif

#ifndef __SERIAL_H
#define __SERIAL_H

/****************************************************************************
*
* GLOBAL HEADER FILES
*
*****************************************************************************/

#include "uart.h"    // class Uart
#include "squeue.h"  // class SerialQueue

/***************************************************************************
*
* CLASS DEFINITION
*
*****************************************************************************/

class Serial : public Uart
{

  /**********************************/

  // Static Data Members

  private:
  static void interrupt (*OldInt0B)(...);  // Old interrupt vector 0x0b
  static void interrupt (*OldInt0C)(...);  // Old interrupt vector 0x0c
  static void interrupt (*OldInt13)(...);  // Old interrupt vector 0x13
  static Boolean portEnabled[];            // COM port enabled flags
  static Boolean wasServiced[];            // COM port serviced flags
  static Serial  *portObject[];            // COM port object pointers
  static unsigned char oldIntMask;         // Old 8259 mask register
  static unsigned char intsInstalled;      // Interrupts hooked flag

  /**********************************/

  // Serial Data Queue Pointers

  protected:
  SerialQueue *rxQueue;  // Received data queue pointer
  SerialQueue *txQueue;  // Transmitted data queue pointer

  /**********************************/

  // Flow Control Data

  private:
  Boolean rxFlowOn;        // Receiver flow on flag
  Boolean txFlowOn;        // Transmitter flow on flag
  unsigned flowType;       // Type of flow control in use
  unsigned char flowChar;  // Next software flow control character to send

  /**********************************/

  // Flow Control Method Selection

  public:
  enum FlowType  // Valid flow control methods
  {
    FLOW_NONE,   // Flow control disabled
    FLOW_HARD,   // Hardware (RTS/CTS) flow control
    FLOW_SOFT    // Software (XON/XOFF) flow control
  };

  /**********************************/

  // Error Accumulator

  unsigned char errorState;

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::AllSent
*
* PURPOSE
*   Test if all data has been sent to the DCE.
*
* DESCRIPTION
*   A true return results ONLY when ALL transmit buffers are empty.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Boolean            // RETURN: True  = All data transmitted
                   //         False = All data not transmitted

Serial::AllSent();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::AnyErrors
*
* PURPOSE
*   Test if any error in reception has occured.
*
* DESCRIPTION
*   All errors, including a break, are abnormal receiver events and are
*   reported by this routine.
*
*   See ClearError() for usage of error test routines.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Boolean              // RETURN: True  = Error has occured
                     //         False = No errors have occured

Serial::AnyErrors();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Break
*
* PURPOSE
*   Transmits a break signal.
*
* DESCRIPTION
*   A break signal is always valid, no matter what the line settings are.
*   This feature is often used to signify bit rate changes, imminent line
*   disconnection, etc. The exact meaning of a break is usage dependent.
*
*   A typical break signal is 250 milliseconds, but this is user selectable
*   by specifing the duration (1-65535 ms).
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

void                 // RETURN: Nothing is returned

Serial::Break
(
  unsigned duration  // INPUT: Duration of break in milliseconds
);

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::BreakDetect
*
* PURPOSE
*   Test if a break signal was received.
*
* DESCRIPTION
*   A break signal is always valid, no matter what the line settings are.
*   This feature is often used to signify bit rate changes, imminent line
*   disconnection, etc. The exact meaning of a break is usage dependent.
*
*   See ClearError() for usage of error test routines.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Boolean                 // RETURN: True  = Break received
                        //         False = Break not received

Serial::BreakDetect();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*  Serial::ClearErrors
*
* PURPOSE
*   Initialize the error history to the 'no error' state.
*
* DESCRIPTION
*   This routine is intended to be called BEFORE transfering data that will
*   be checked for errors. Since received data is always queued, errors
*   cannot be detected on a character by character basis. Instead, errors
*   can only be detected between calls to this routine.
*
*   All errors, including breaks, result in a loss of data that is NOT put
*   in the receive queue. For most types of communications, errors can be
*   ignored completely or are handled by higher level protocols. Called by
*   the constructor.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

void                    // RETURN: Nothing is returned

Serial::ClearErrors();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Disable
*
* PURPOSE
*   Disables a serial port.
*
* DESCRIPTION
*   This routine was designed to allow the queues to be resized by calling
*   Enable() afterwards. Called by the destructor.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

void                // RETURN: Nothing is returned

Serial::Disable();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Enable
*
* PURPOSE
*   Enable a serial port for use.
*
* DESCRIPTION
*   This routine was designed to allow the queues to be resized by first
*   calling Disable(). Queues from 256 to 65520 characters can be used.
*   Called by the constructor.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Boolean              // RETURN: True  = Enable succeeded
                     //         False = Enabled failed

Serial::Enable
(
  unsigned rxQSize,  // INPUT: Size of receive queue (in characters)
  unsigned txQSize   // INPUT: Size of transmit queue (in characters)
);

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::EnableInt
*
* PURPOSE
*   Enable serial interrupts, preserving the original setting.
*
* DESCRIPTION
*   This routine simply clears the 8259 PIC mask bits for IRQ3 and IRQ4,
*   allowing them to be recognized by the processor.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

private:

void                  // RETURN: Nothing is returned

Serial::EnableInt();

/*****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::FrameError
*
* PURPOSE
*   Test if a frame error in reception has occured.
*
* DESCRIPTION
*   Frame errors indicate a missing stop bit. These errors are generally
*   caused by line noise.
*
*   See ClearError() for usage of error test routines.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Boolean               // RETURN: True  = Frame error detected
                      //         False = Frame error not detected

Serial::FrameError();

/****************************************************************************
*
* FILENAME
*   SERIAL.H
*
* FUNCTION NAME
*   Serial::GetRxCount
*
* PURPOSE
*   Return the number of bytes in the receive queue.
*
* DESCRIPTION
*   The value returned is the MINIMUM number of bytes in the queue, as it is
*   possible that additional bytes arrived before the function returned.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

unsigned              // RETURN: Bytes in the receive queue

Serial::GetRxCount()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return the number of bytes in the receive queue
  return rxQueue->GetCount();
}

/****************************************************************************
*
* FILENAME
*   SERIAL.H
*
* FUNCTION NAME
*   Serial::GetTxCount
*
* PURPOSE
*   Return the number of bytes in the transmit queue.
*
* DESCRIPTION
*   The value returned is the MAXIMUM number of bytes in the queue, as it is
*   possible that additional bytes were sent before the function returned.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

unsigned              // RETURN: Bytes in the transmit queue

Serial::GetTxCount()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return the number of bytes in the trasmit queue
  return txQueue->GetCount();
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::HandleError
*
* PURPOSE
*   Interrupt routine to handle uart error conditions, or a break.
*
* DESCRIPTION
*   Data in error is not queued; this ensures the queue is always valid. If
*   character sequencing is critical, the occurance of an error can be
*   detected by AnyErrors(). See ClearError() for details on errors.
*
*   Note that this function is private and inline!
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

private:

void                   // RETURN: Nothing is returned

Serial::HandleError();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::HandleMSR
*
* PURPOSE
*   Interrupt routine to handle DCE signal changes.
*
* DESCRIPTION
*   Hardware flow control is handled by this routine. Since any change in any
*   of the DCE signals will cause this routine to be called, the MSR data
*   member is always valid. This is an added bonus; it is much faster to
*   check a memory location, than it is to access a potentially slow bus
*   device.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

private:

void                 // RETURN: Nothing is returned

Serial::HandleMSR();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::HandleRDR
*
* PURPOSE
*   Interrupt routine to handle received data.
*
* DESCRIPTION
*   When flow control is enabled, it is exercised by this routine for the
*   receiver. If the DCE does not honor a request to stop transmitting
*   quickly, data will be lost but no OverflowError() will be reported.
*
*   Note that this function is private and inline!
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

private:

void

Serial::HandleRDR();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::HandleTHR
*
* PURPOSE
*   Interrupt routine to handle transmitted data.
*
* DESCRIPTION
*   Transmmitter interrupts will be disabled if their is nothing to transmit,
*   or if the transmitter flow has been disabled by flow control.
*
*   Note that this function is private and inline!
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

private:

void                 // RETURN: Nothing is returned

Serial::HandleTHR();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::HandleInt
*
* PURPOSE
*   Main interrupt routine that decides what, if anything, needs handling.
*
* DESCRIPTION
*   Checks interrupt enable bits before handling interrupts, since the ID
*   register doesn't have a 'no interrupt' state. This prevents accidentally
*   handling someone else's interrupt.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

private:

Boolean              // RETURN: True  = Interrupt was handled
                     //         False = Interrupt was not handled

Serial::HandleInt();

/*****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::In
*
* PURPOSE
*   Input a character from an enabled serial port.
*
* DESCRIPTION
*   This routine does NOT check if the port is enabled, and can potentially
*   hang the controlling program in a busy-wait state. To avoid delays,
*   make sure the port is enabled, and test a call to IsRxEmpty().
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

unsigned char  // RETURN: Character from receive queue

Serial::In();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::InstallInt
*
* PURPOSE
*   Install serial interrupt handlers, preserving the original vectors.
*
* DESCRIPTION
*   The interrupt handlers are installed for all four ports if they have not
*   already been installed.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

private:

void                  // RETURN: Nothing is returned

Serial::InstallInt();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Int0B
*
* PURPOSE
*   Interrupt routine for COM2 and COM4 (IRQ 3).
*
* DESCRIPTION
*   Identical to COM1 and COM3 routine, in all but the ports handled.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

private: static

void           // RETURN: Nothing is returned

interrupt      // This is an interrupt routine

Serial::Int0B
(
  ...          // INPUT: Interrupt stack frame (unused)
);

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Int0C
*
* PURPOSE
*   Interrupt routine for COM1 and COM3 (IRQ 4).
*
* DESCRIPTION
*   Identical to COM2 and COM4 routine, in all but the ports handled.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

private: static

void            // RETURN: Nothing is returned

interrupt       // This is an interrupt routine

Serial::Int0C
(
  ...           // INPUT: Interrupt stack frame (unused)
);

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Int13
*
* PURPOSE
*   Interrupt routine for disk access (INT 0x13) flow control.
*
* DESCRIPTION
*   This routine turns off the data flow to all serial ports whenever a disk
*   access is detected. The data flow is restored when the disk access is
*   completed.
*
*   THIS IS EXPERIMENTAL!
*
*   This function slows disk accesses, and may not detect accesses either.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

private: static

void              // RETURN: Nothing is returned

interrupt         // This is an interrupt routine

Serial::Int13
(
  unsigned bp,    // INPUT: Base pointer register
  unsigned di,    // INPUT: Destination index register
  unsigned si,    // INPUT: Source index register
  unsigned ds,    // INPUT: Data segment register
  unsigned es,    // INPUT: Extra segment register
  unsigned dx,    // INPUT: DX register
  unsigned cx,    // INPUT: CX register
  unsigned bx,    // INPUT: BX register
  unsigned ax,    // INPUT: AX rexister
  unsigned ip,    // INPUT: Instruction pointer register
  unsigned cs,    // INPUT: Code segment register
  unsigned flags  // INPUT: FLAGS register
);

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::IsEnabled
*
* PURPOSE
*   Check if the port is enabled.
*
* DESCRIPTION
*   It is wise to check if a port was successfully enabled before using it.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Boolean              // RETURN: True  = Port is enabled
                     //         False = Port is disabled

Serial::IsEnabled()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return the enabled state of the port
  return portEnabled[ portNumber ];
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::NoCarrier
*
* PURPOSE
*   Get the status of the data carrier detect (DCD) signal.
*
* DESCRIPTION
*   This is a very fast test that does not acesss the physical port. It is
*   always correct, thanks to the HandleMSR() routine called by the interrupt
*   handler.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Boolean               // RETURN: True  = DCD is asserted
                      //         False = DCD is negated

Serial::NoCarrier();

/*****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Out
*
* PURPOSE
*   Output a character to an enabled serial port.
*
* DESCRIPTION
*   This routine does NOT check if the port is enabled, and can potentially
*   hang the controlling program in a busy-wait state. To avoid delays,
*   make sure the port is enabled, and test a call to IsTxFull();
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

void                  // RETURN: Nothing is returned

Serial::Out
(
  unsigned char data  // INPUT: Character to output to the serial port
);

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::OverflowError
*
* PURPOSE
*   Test if an overflow error in reception has occured.
*
* DESCRIPTION
*   This error is serious; it means the interrupt handler did not react
*   quickly enough, and data was lost. The fault may not be with the
*   interrupt handler, since DMA and servicing other interupts can delay
*   servicing a port. Should you detect this error, please contact the
*   author.
*
*   See ClearError() for usage of error test routines.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Boolean                   // RETURN: True  = Overflow detected
                          //         False = Overflow not detected

Serial::OverflowError();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::ParityError
*
* PURPOSE
*   Test if a parity error in reception has occured.
*
* DESCRIPTION
*   Parity errors are only valid when PARITY_ODD, or PARITY_EVEN is used.
*   These errors are generally caused by line noise. Parity is not a secure
*   error check method, since it is easily fooled.
*
*   See ClearError() for usage of error test routines.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Boolean                   // RETURN: True  = Parity error detected
                          //         False = Parity error not detected

Serial::ParityError();

/****************************************************************************
*
* FILENAME
*   SERIAL.H
*
* FUNCTION NAME
*   Serial::Peek
*
* PURPOSE
*   Peek at the contents of the receive queue.
*
* DESCRIPTION
*   The queue is not altered in any way. Attempts to Peek beyond the data
*   in the queue always return ASCII NULL.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

unsigned char      // RETURN: byte in queue

Serial::Peek
(
  unsigned offset  // INPUT: Offset of byte from head of queue
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Call Peek for the received data queue
  return rxQueue->Peek( offset );
}

/****************************************************************************
*
* FILENAME
*   SERIAL.H
*
* FUNCTION NAME
*   Serial::PurgeRx
*
* PURPOSE
*   Purge the receive queue.
*
* DESCRIPTION
*   The queue is reset to the empty state; the contents are lost.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void               // RETURN: Nothing is returned

Serial::PurgeRx()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Purge the received data queue
  rxQueue->Purge();
}

/****************************************************************************
*
* FILENAME
*   SERIAL.H
*
* FUNCTION NAME
*   Serial::PurgeTx
*
* PURPOSE
*   Purge the transmit queue.
*
* DESCRIPTION
*   The queue is reset to the empty state; the contents are lost.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void               // RETURN: Nothing is returned

Serial::PurgeTx()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Purge the transmitted data queue
  txQueue->Purge();
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::RemoveInt
*
* PURPOSE
*   Remove serial interrupt handlers, restoring the original vectors.
*
* DESCRIPTION
*   The vectors are only removed if they are installed, and none of the four
*   possible ports is enabled.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

private:

void                  // RETURN: Nothing is returned

Serial::RemoveInt();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::RestoreInt
*
* PURPOSE
*
*   Restore serial interrupts to their original settings.
*
* DESCRIPTION
*
*   This routine simply resets the 8259 PIC mask bits for IRQ3 and IRQ4 to
*   their original settings.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

private:

void                   // RETURN: Nothing is returned

Serial::RestoreInt();

/****************************************************************************
*
* FILENAME
*   SERIAL.H
*
* FUNCTION NAME
*   Serial::RxIsEmpty
*
* PURPOSE
*   Test if the receive queue is empty.
*
* DESCRIPTION
*   If the queue is empty True is returned, otherwise False is returned.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Boolean              // RETURN: True  = Receive queue empty
                     //         False = Receive queue not empty

Serial::RxIsEmpty()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return the status of the received data queue
  return rxQueue->IsEmpty();
}

/****************************************************************************
*
* FILENAME
*   SERIAL.H
*
* FUNCTION NAME
*   Serial::RxIsFull
*
* PURPOSE
*   Test if the receive queue is full.
*
* DESCRIPTION
*   If the queue is full True is returned, otherwise False is returned.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Boolean              // RETURN: True  = Receive queue full
                     //         False = Receive queue not full

Serial::RxIsFull()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return the state of the received data queue
  return rxQueue->IsFull();
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Serial
*
* PURPOSE
*   Copy constructor for class Serial.
*
* DESCRIPTION
*   This function is declared, but not defined. This is done so that copying
*   will not take place.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Serial::Serial
(
  Serial &original  // INPUT: Serial object to copy
);

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Serial
*
* PURPOSE
*   Constructor for class Serial.
*
* DESCRIPTION
*   Any one of the four COM ports can be used, but a port cannot have more
*   than one Serial object associated with it. This limitation is logical,
*   and is a result of the static nature of the interrupt routines. All other
*   parameters can be adjusted by using the appropriate member functions.
*
*   This constructor does not enable the port; see Enable() for details.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Serial::Serial
(
  Serial::ComPort comPort  // INPUT: Serial port to use
);

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::Serial
*
* PURPOSE
*   Constructor for class Serial.
*
* DESCRIPTION
*   Any one of the four COM ports can be used, but a port cannot have more
*   than one Serial object associated with it. This limitation is logical,
*   and is a result of the static nature of the interrupt routines. All other
*   parameters can be adjusted by using the appropriate member functions.
*
*   This constructor enables the port; see Enable() for details.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Serial::Serial
(
  Serial::ComPort comPort,   // INPUT: Serial port to use
  unsigned rxQSize,          // INPUT: Size of receive queue (characters)
  unsigned txQSize           // INPUT: Size of transmit queue (characters)
);

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::~Serial
*
* PURPOSE
*   Destructor for class Serial.
*
* DESCRIPTION
*   The port is disabled, which removes all queues and possibly removes the
*   interrupt handlers.
*
*   Once destroyed, the COM port is completely ignored.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: virtual

Serial::~Serial();

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::SetComPort
*
* PURPOSE
*   Change the COM port used
*
* DESCRIPTION
*   This function can be called at anytime.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

void                       // RETURN: Nothing is returned

Serial::SetComPort
(
  Serial::ComPort comport  // INPUT: COM port to use
);

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::SetFlowType
*
* PURPOSE
*   Set the flow control method.
*
* DESCRIPTION
*   Hardware (FLOW_HARD) using RTS/CTS signals, software (FLOW_SOFT) using
*   XON/XOFF characters, or none (FLOW_NONE) methods of flow control can be
*   selected. Calling this routine resets the present flow control state,
*   and therefore it is best to call after creating a serial port object
*   (always created with FLOW_NONE), or after disabling an active port.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

void                     // RETURN: Nothing is returned

Serial::SetFlowType
(
  Serial::FlowType flow  // INPUT: Flow control method to use
);

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::SetRxFlowOn
*
* PURPOSE
*   Send a flow control request to the DCE.
*
* DESCRIPTION
*   This routine is intended to be used to manually control the receiver's
*   flow control.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

void                 // RETURN: Nothing is returned

Serial::SetRxFlowOn
(
  Boolean state      // INPUT: True  = Flow control asserted
                     //        False = Flow control negated
);

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::SetTxFlowOn
*
* PURPOSE
*   Impose transmitter flow control by enabling/disabling the transmitter.
*
* DESCRIPTION
*   This routine both changes the transmitter flow setting, as well as
*   enable/disable the transmitter.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

void                 // RETURN: Nothing is returned

Serial::SetTxFlowOn
(
  Boolean state      // INPUT: True  = Flow control asserted
                     //        False = Flow control negated
);

/****************************************************************************
*
* FILENAME
*   SERIAL.H
*
* FUNCTION NAME
*   Serial::TxIsEmpty
*
* PURPOSE
*   Test if the transmit queue is empty.
*
* DESCRIPTION
*   If the queue is empty True is returned, otherwise False is returned.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Boolean              // RETURN: True  = Transmit queue is empty
                     //         False = Transmit queue is not empty

Serial::TxIsEmpty()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return the state of the transmit queue
  return txQueue->IsEmpty();
}

/****************************************************************************
*
* FILENAME
*   SERIAL.CPP
*
* FUNCTION NAME
*   Serial::TxIsFull
*
* PURPOSE
*   Test if the transmit queue is full.
*
* DESCRIPTION
*   If the queue is full True is returned, otherwise False is returned.
*
* EXTERNAL REFERENCES
*   enum  Boolean
*   class Serial
*   class SerialQueue
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SQUEUE.OBJ
*     UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Boolean             // RETURN: True  = Transmit queue is full
                    //         False = Transmit queue is not full

Serial::TxIsFull()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return the state of the transmint queue
  return txQueue->IsFull();
}

/****************************************************************************
*
* CLASS END
*
*****************************************************************************/

};

/****************************************************************************
*
* PREPROCESSOR EXCLUSION END
*
*****************************************************************************/

#endif
