/****************************************************************************
*
* FILENAME
*   SQUEUE.CPP
*
* DESCRIPTION
*   Out-of-line functions for class SerialQueue.
*
* CONTENTS
*   unsigned char SerialQueue::GetData      ()
*   unsigned char SerialQueue::Peek         (unsigned)
*   void          SerialQueue::PutData      (unsigned char)
*                 SerialQueue::SerialQueue  (unsigned, unsigned, unsigned)
*                 SerialQueue::~SerialQueue ()
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* GLOBAL HEADER FILES
*
*****************************************************************************/

#include "squeue.h"  // class SerialQueue
#include "ascii.h"   // ASCII definitions
#include <_null.h>   // Definition of NULL

/****************************************************************************
*
* FILENAME
*   SQUEUE.CPP
*
* FUNCTION NAME
*   SerialQueu::GetData
*
* PURPOSE
*   Remove and return a byte from the queue.
*
* DESCRIPTION
*   If the queue is already empty, then an ASCII NULL is returned.
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

unsigned char           // RETURN: Byte removed from queue

SerialQueue::GetData()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // If queue empty, then return an ASCII NULL.
  unsigned char data = ASCII_NUL;

  // Remove a byte from queue if it exists and it's not empty
  if( size && count > 0 )
  {
    data = queue[head];

    // Addjust queue indicies
    if( head < ( size - 1 ))
    {
      head++;
    }
    else
    {
      head = 0;
    }

    // Decrement queue count
    count--;
  }

  return data;
}

/****************************************************************************
*
* FILENAME
*   SQUEUE.CPP
*
* FUNCTION NAME
*   SerialQueue::Peek
*
* PURPOSE
*   Peek at the contents of the queue.
*
* DESCRIPTION
*   The queue is not altered in any way. Attempts to Peek beyond the data
*   in the queue always return ASCII NULL.
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

unsigned char      // RETURN: requested byte in queue

SerialQueue::Peek
(
  unsigned offset  // INPUT: Offset of byte from head of queue
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  unsigned char retval;
  unsigned noWrap;

  // If offset is valid, peek into queue
  if( offset <= count )
  {
    noWrap = size - head ;

    if( offset < noWrap )
    {
      offset += head;
    }
    else
    {
      offset -= noWrap;
    }

    // Peek into queue
    retval = queue[ offset ];
  }
  else
  {
    // Invalid offset specified
    retval = ASCII_NUL;
  }
  // Return the peeked value
  return retval;
}

/****************************************************************************
*
* FILENAME
*   SQUEUE.CPP
*
* FUNCTION NAME
*   SerialQueue::PutData
*
* PURPOSE
*   Put a byte in the queue.
*
* DESCRIPTION
*   If the queue is full already, then the byte is simply ignored (ie. lost).
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                  // RETURN: Nothing is returned.

SerialQueue::PutData
(
  unsigned char data  // INPUT: Byte to put into the queue
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Insert byte into queue if it is valid, and there is room for it
  if( size && count < size )
  {
    queue[ tail ] = data;

    // Adjust queue indicies
    if( tail < ( size - 1 ))
    {
      tail++;
    }
    else
    {
      tail = 0;
    }

    // Increment queue count
    count++;
  }
}

/****************************************************************************
*
* FILENAME
*   SQUEUE.CPP
*
* FUNCTION NAME
*   SerialQueue::SerialQueue
*
* PURPOSE
*   Constructor for class SerialQueue.
*
* DESCRIPTION
*   The size of the queue must be specified; this is the TOTAL number of
*   bytes that will be allocated. The minimum queue size is 256, and the
*   maximum is approximately 65520.
*
*   The trigger levels for NearFull() and NearEmpty() are set by topSpace
*   and middleSpace respectively.
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E.Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

SerialQueue::SerialQueue
(
  unsigned qSize,         // INPUT: Size of queue in bytes
  unsigned topSpace,      // INPUT: Trigger level for NearFull()
  unsigned middleSpace    // INPUT: Trigger level for NearEmpty()
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Force minimum size of queue to 256 bytes
  size = ( qSize > 256 ) ? qSize : 256;

  // Try to allocate queue
  if(( queue = new unsigned char[ size ] ) == NULL )
  {
    // Queue allocation failed, so allocate minimum size queue
    queue = new unsigned char[256];
  }

  // If allocation successful, set queue variables
  if(queue != NULL)
  {
    // Set NearFull() level
    full  = ( topSpace > size ) ? size - 50: size - topSpace;

   // Set NearEmpty() level
    empty = ( middleSpace > full ) ? full - 50: full - middleSpace;
  }
  else
  {
    // Allocation failed, so queue is invalid
    full = empty = size = 0;
  }

  // Initialize queue variables
  Purge();

}

/****************************************************************************
*
* FILENAME
*   SQUEUE.CPP
*
* FUNCTION NAME
*   SerialQueue::~SerialQueue
*
* PURPOSE
*   Destructor for class SerialQueue.
*
* DESCRIPTION
*   Memory allocated by the constructor is deallocated.
*
* EXTERNAL REFERENCES
*   class SerialQueue
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   SQUEUE.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

SerialQueue::~SerialQueue()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // If memory was allocated, then deallocate it
  if(queue != NULL)
  {
    delete [] queue;
  }
}

/****************************************************************************/
