/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* DESCRIPTION
*   Essential external definitions for the classes SerialBuf and SerialStream.
*
* CONTENTS
*   int    SerialBuf::overflow         (int)
*          SerialBuf::SerialBuf        (Serial *)
*   int    SerialBuf::underflow        ()
*
*          SerialStream::SerialStream  (Serial::ComPort)
*          SerialStream::SerialStream  (Serial::ComPort, unsigned, unsigned)
*          SerialStream::~SerialStream ()
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* GLOBAL HEADER FILES
*
*****************************************************************************/

#include "sstream.h"  // class SerialBuf and class SerialStream

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialBuf::overflow
*
* PURPOSE
*   Called whenever the internal buffer overflows.
*
* DESCRIPTION
*   Since there is no internal buffer, the buffer overflows on every output
*   operation. The byte that caused the overflow is put in the queue for the
*   serial transmitter.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialBuf
*   class streambuf
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

int                   // RETURN: EOF if operation failed (never!)

SerialBuf::overflow
(
  int c               // INPUT: Byte that caused the overflow
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Extract the overflow character
  c &= 0xff;

  // Reset the internal buffer pointers
  setp( buf + 1, buf + 1 );

  // Output the overflow byte to the serial port
  serialObject->Out( c );

  // Return the overflow character
  return c;
}

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialBuf::SerialBuf
*
* PURPOSE
*   Constructor for class SerialBuf.
*
* DESCRIPTION
*   A pointer to a class Serial object to be used is required.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialBuf
*   class streambuf
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

SerialBuf::SerialBuf
(
  Serial *serialPort  // INPUT: Pointer to class Serial object
)

/****************************************************************************
*
* BASE CLASS AND CONSTANT INITIALIZATIONS
*
*****************************************************************************/

: streambuf()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Link to a class serial object
  serialObject = serialPort;

  // Set internal buffer area
  setb( buf, buf + 2, 0 );

  // Set up an internal buffered get area of 1 byte
  setg( buf, buf + 1, buf + 1 );

  // Set up an internal unbuffered put area of 0 bytes
  setp( buf + 1, buf + 1 );
}

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialBuf::underflow
*
* PURPOSE
*   Called whenever the internal buffer underflows.
*
* DESCRIPTION
*   Since there is no internal buffer will, it will underflow on every input
*   operation. A byte is removed from the serial rece1ver queue, and
*   returned as if it had come from an internal buffer.
*
* EXTERNAL REFERENCES
*   class Serial
*   class SerialBuf
*   class streambuf
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

int                     // RETURN: EOF if operation failed (never!)

SerialBuf::underflow()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Reset the internal buffer pointers
  setg(buf, buf, buf + 1);

  // Get a byte from the serial port and return it
  return *buf = serialObject->In();
}

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialStream::SerialStream
*
* PURPOSE
*   Constructor that does not enable port.
*
* DESCRIPTION
*   The port must be enabled before it can be used.
*
* EXTERNAL REFERENCES
*   class SerialBuf
*   class SerialStream
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

SerialStream::SerialStream
(
  SerialStream::ComPort comPort  // INPUT: COM port to be used
)

/****************************************************************************
*
* BASE CLASS AND CONSTANT INITIALIZATIONS
*
*****************************************************************************/

: Serial(comPort),
  iostream()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Create a SerialBuf
  serialBuf = new SerialBuf(this);

  // Initialize class ios
  ios::init(serialBuf);

  // No whitespace skipping
  unsetf(skipws);
}

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialStream::SerialStream
*
* PURPOSE
*   Constructor that enables port.
*
* DESCRIPTION
*   The port is ready for use after it is constructed.
*
* EXTERNAL REFERENCES
*   class SerialBuf
*   class SerialStream
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

SerialStream::SerialStream
(
  ComPort comPort,   // INPUT: COM port to use
  unsigned rxQSize,  // INPUT: Size of receive queue
  unsigned txQSize   // INPUT: Size of transmit queue
)

/****************************************************************************
*
* BASE CLASS AND CONSTANT INITIALIZATIONS
*
*****************************************************************************/

: Serial(comPort, rxQSize, txQSize),
  iostream()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Create a SerialBuf
  serialBuf = new SerialBuf(this);

  // Initialize class ios
  ios::init(serialBuf);

  // No whitespace skipping
  unsetf(skipws);
}

/****************************************************************************
*
* FILENAME
*   SSTREAM.CPP
*
* FUNCTION NAME
*   SerialStream::~SerialStream
*
* PURPOSE
*   Destructor for the class.
*
* DESCRIPTION
*   The internal SerialBuf is deallocated.
*
* EXTERNAL REFERENCES
*   class SerialBuf
*   class SerialStream
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   SERIAL.OBJ
*   SSTREAM.OBJ
*   SQUEUE.OBJ
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

SerialStream::~SerialStream()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // If a class SerialBuf is owned, then delete it
  if(!serialBuf)
  {
    delete serialBuf;
    serialBuf = NULL;
  }
}

/****************************************************************************/


