// --------------------------------------------------------------------
//
//   Construct a CDialog template
//
//   Copyright 1993, Chris Shearer Cooper  CompuServe 76666,3026
//   This file may be copied and distributed free of charge as long as
//   this copyright notice remains intact and any changes by other
//   authors are clearly indicated.
//   This code may be used in your application without any monetary
//   royalty, however payment is required in the form of feedback and
//   bug reports to the author.  Although every effort has been made
//   to make this code bug-free, if you use this code in your application
//   the author takes no responsibility for any damages which may occur
//   from the operation of this code.
//
// --------------------------------------------------------------------

#ifndef __TEMPLATE_H

#include "classes.h"

#include "tresourc.h"
    
#include <limits.h>

class CDialogTemplate;

class CDialogItem : public CObject
	{
public:
	__export CDialogItem(CDialogTemplate *pTemplate, LPCSTR szClass, long lStyle);
	__export CDialogItem(CDialogTemplate *pTemplate, unsigned char cClass, long lStyle = LONG_MIN);
	__export CDialogItem(CDialogTemplate *pTemplate, const char *pData, int *piBytesUsed);
	__export CDialogItem(CDialogTemplate *pTemplate, const CDialogItem *pSrcItem);

	// Setting focus : if no priority is specified, defaults to 1.  Positive
	// priorities are ranked in reverse order (priority 1 wins over priority
	// 2) and always win over negative or zero priorities.  If multiple items
	// have the winning positive priority value, the first item defined (not
	// necessarily the first item to have set_focus() called) will win.
	// Nonpositive (zero and negative) priorities are compared using their
	// absolute values : priority 0 wins over priority -1 which wins over
	// priority -2.  If no positive priority values are found, and multiple
	// items have the winning nonpositive priority value, the last item
	// defined (not necessarily the last item to have set_focus() called) will
	// win.  The function get_focus() in CDialogTemplate must be called
	// to determine the winner.
	CDialogItem * __export set_focus(int iPriority = 1)
		{ m_iFocusPriority = iPriority; return this; }
	
	CDialogItem *set_position(int iLeft, int iTop)
		{ set_left(iLeft); return set_top(iTop); }
	CDialogItem *set_position(CPoint &pt)
		{ set_left(pt.x); return set_top(pt.y); }
	CDialogItem *set_left(int iLeft)
		{
		if (m_rect.right >= 0)
			m_rect.right += iLeft - m_rect.left;
		m_rect.left = iLeft;
		return this;
		}
	CDialogItem *set_top(int iTop)
		{
		if (m_rect.bottom >= 0)
			m_rect.bottom += iTop - m_rect.top;
		m_rect.top = iTop;
		return this;
		}
	CPoint __export get_position(void);

	CDialogItem * __export set_size(int iWidth, int iHeight);
	CDialogItem *set_width(int iWidth)
		{ m_rect.right = m_rect.left + iWidth; return this; }
	CDialogItem *set_height(int iHeight)
		{ m_rect.bottom = m_rect.top + iHeight; return this; }
	CDialogItem *set_right(int iRight)
		{ m_rect.right = iRight; return this; }
	CDialogItem *set_bottom(int iBottom)
		{ m_rect.bottom = iBottom; return this; }

	CRect get_rect(void) const
		{ return m_rect; }
	void set_rect(const RECT &rect)
		{ m_rect = rect; }

	long get_style(void) const
		{ return m_lStyle; }
	void set_style(long lStyle)
		{ m_lStyle = lStyle; return; }

	// set_text() will set the size (if not already set)
	CDialogItem * __export set_text(LPCSTR szText);
	CDialogItem *set_text(UINT uResourceID)
		{ CString szText; szText.LoadString(uResourceID); return set_text((LPCSTR)szText); }
	
	CDialogItem *set_id(unsigned int uID)
		{ m_uID = uID; return this; }
	unsigned int get_id(void) const
		{ return m_uID; }

	BOOL __export generate(CFile &hFile);

	int right(void) const	{ return m_rect.right; }
	int left(void) const	{ return m_rect.left; }
	int top(void) const		{ return m_rect.top; }
	int bottom(void) const	{ return m_rect.bottom; }
	int width(void) const	{ return m_rect.Width(); }
	int height(void) const	{ return m_rect.Height(); }
				
protected:
	CRect m_rect;			// In dialog box units!
	unsigned int m_uID;
	long m_lStyle;
	int m_iFocusPriority;
	CString m_szClass;
	CString m_szText;
	CDialogTemplate *m_pTemplate;
	};
	

class CDialogTemplate : public CObject
	{
public:
	__export CDialogTemplate(CWnd *pParent);
	__export ~CDialogTemplate();
	
	BOOL __export load(LPCSTR szTemplateName);
	BOOL __export load(int iTemplateID);
	BOOL __export load(tResource &hResource);
	
	CDialogTemplate &set_menu(LPCSTR szMenuName)
		{ m_szMenuName = szMenuName; return *this;}
	CDialogTemplate &set_class(LPCSTR szClassName)
		{ m_szClassName = szClassName; return *this; }
	CDialogTemplate &set_style(long lStyle)
		{ m_lStyle = lStyle; return *this; }
	BOOL __export set_font(LPCSTR szFaceName, WORD wPointSize);

	CDialogTemplate &set_caption(LPCSTR szCaption)
		{ m_szCaption = szCaption; return *this; }
	CDialogTemplate &set_caption(UINT nID)
		{ m_szCaption.LoadString(nID); return *this; }
	LPCSTR get_caption(void) const
		{ return (LPCSTR)m_szCaption; }

	// Call set_position() _before_ set_size()
	CDialogTemplate &set_position(int iLeft, int iTop)
		{ m_rect.left = iLeft; m_rect.top = iTop; return *this; }
	CDialogTemplate & __export set_size(int iWidth, int iHeight);
	CSize __export get_size(void);
	
	CDialogItem * __export add_item(LPCSTR szClass, long lStyle);
	CDialogItem * __export add_item(unsigned char cClass, long lStyle = LONG_MIN);
	// Notice that this next function makes a copy of the given item and
	// keeps hold of that.  If iIndex is specified, inserts the new
	// item in the array at the specified index, moving all other items
	// up a slot.
	CDialogItem * __export add_item(const CDialogItem *pItemToCopy, int iIndex = -1);

	void * __export generate(UINT *puTemplateSize = NULL);

	// Dialog values are in dialog units; pixel values are in client window units
	int __export pixel_x_to_dialog(int iPixelX);
	int __export pixel_y_to_dialog(int iPixelY);
	CSize __export pixel_to_dialog(const CSize &size);
	CRect __export pixel_to_dialog(const CRect &rect);
	int __export dialog_x_to_pixel(int iDialogX);
	int __export dialog_y_to_pixel(int iDialogY);
	CSize __export dialog_to_pixel(const CSize &size);
	CRect __export dialog_to_pixel(const CRect &rect);

	CDC *get_dc(void)
		{ return m_pDC; }

	int num_items(void) const
		{ return m_aItems.GetSize(); }
	CDialogItem *get_item(int iIndex)
		{ return (CDialogItem *)m_aItems[iIndex]; }
	CDialogItem * __export find_item(UINT uID);
	int __export find_item_index(UINT uID);

	BOOL __export remove_item(UINT uID);

	// Returns an ID which does not currently exist in the template
	UINT __export unused_id(void);

protected:	
	long m_lStyle;
	CRect m_rect;
	CString m_szMenuName;
	CString m_szClassName;
	CString m_szCaption;
	CObArray m_aItems;
	WORD m_wPointSize;
	CString m_szFaceName;
	long m_lDialogUnits;
	CWnd *m_pParent;
	CClientDC *m_pDC;
	CFont *m_pFont;
	};
	
#define __TEMPLATE_H
#endif
