/****************************************************************************
*
* FILENAME
*   UART.CPP
*
* DESCRIPTION
*   Essential external definitions for the class Uart.
*
* CONTENTS
*   void Uart::GetAllRegs ()
*   void Uart::SetBitRate (unsigned long)
*   void Uart::SetComPort (Uart::ComPort)
*
*   unsigned Uart::comAddress[]
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* GLOBAL HEADER FILES
*
*****************************************************************************/

#include "uart.h"  // class Uart

/****************************************************************************
*
* STATICS
*
*****************************************************************************/

unsigned Uart::comAddress[] = { 0x3f8, 0x2f8, 0x3e8, 0x2e8 };

/****************************************************************************
*
* FILENAME
*   UART.CPP
*
* FUNCTION NAME
*   Uart::GetAllRegs
*
* PURPOSE
*   Set all the UART shadow registers to the current values.
*
* DESCRIPTION
*   The UART is read to set each of the shadow registers. Considering that
*   the UART is on the I/O bus, this is a VERY slow operation. Using the
*   shadow registers is much faster than using the actual registers.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup and Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. MIleski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                // RETURN: Nothing is returned

Uart::GetAllRegs()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Initialize all the UART shadow registers
  GetLSR();
  GetMSR();
  GetMCR();
  GetLCR();
  GetIIR();
  SetRegSet( True );
  GetBRD();
  SetRegSet( False );
  GetIER();
  GetRDR();
}

/****************************************************************************
*
* FILENAME
*   UART.CPP
*
* FUNCTION NAME
*   Uart::SetBitRate
*
* PURPOSE
*   Set the bit rate used by the UART.
*
* DESCRIPTION
*   The bit rate can be rate between 1 and 115200 bits per second.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                     // RETURN: Nothing returned

Uart::SetBitRate
(
  unsigned long bitRate  // INPUT: Bit rate (bits per second)
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // If bit rate is valid, then set the UART bit rate divisor (BRD)
  if( bitRate > 1 && bitRate <= 115200L )
  {
    BRD = 115200L / bitRate;
    SetRegSet( True );
    SetBRD();
    SetRegSet( False );
  }

}

/****************************************************************************
*
* FILENAME
*   UART.CPP
*
* FUNCTION NAME
*   Uart::SetComPort
*
* PURPOSE
*   Set the COM port used.
*
* DESCRIPTION
*   The COM port can be safely changed at any time with this function.
*   All the registers of the UART are read to initialize the shadow
*   registers.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

void                     // RETURN: Nothing is returned

Uart::SetComPort
(
  Uart::ComPort comPort  // INPUT: COM port to use
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Save the COM port number
  portNumber  = comPort;

  // Get the base address of the UART
  baseAddress = comAddress[ comPort ];

  // Initialize shadow registers
  GetAllRegs();
}

/****************************************************************************/
