/****************************************************************************
*
* FILENAME
*   UART.H
*
* CLASSNAME
*   Uart
*
* PURPOSE
*   Register level 8250 UART manipulations.
*
* DESCRIPTION
*   This class was specifically designed to be the base of class Serial.
*
* INHERITANCE
*   None
*
* MEMBER FUNCTIONS
*   public:
*   unsigned long Uart::GetBitRate   ()
*   void          Uart::SetComPort   ()
*   DataBits      Uart::GetDataBits  ()
*   ParityBit     Uart::GetParityBit ()
*   StopBits      Uart::GetStopBits  ()
*   void          Uart::SetBitRate   (unsigned long)
*   void          Uart::SetDataBits  ()
*   void          Uart::SetParityBit ()
*   void          Uart::SetStopBits  ()
*                 Uart::Uart         (Uart &)
*                 Uart::Uart         (Uart::Comport)
*                 Uart::~Uart        ()
*
*   protected:
*   void          Uart::GetAllRegs   ()
*   void          Uart::GetBRD       ()
*   Boolean       Uart::GetCTS       ()
*   Boolean       Uart::GetDCD       ()
*   Boolean       Uart::GetDSR       ()
*   void          Uart::GetIER       ()
*   void          Uart::GetIIR       ()
*   void          Uart::GetLCR       ()
*   void          Uart::GetLSR       ()
*   void          Uart::GetMCR       ()
*   void          Uart::GetMSR       ()
*   unsigned char Uart::GetRDR       ()
*   Boolean       Uart::GetRI        ()
*   void          Uart::SetBRD       ()
*   void          Uart::SetDTR       (Boolean)
*   void          Uart::SetIER       ()
*   void          Uart::SetLCR       ()
*   void          Uart::SetMCR       ()
*   void          Uart::SetRegSet    (Boolean)
*   void          Uart::SetRTS       (Boolean)
*   void          Uart::SetTHR       ()
*   void          Uart::SetUser1     (Boolean)
*   void          Uart::SetUser2     (Boolean)
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PREPROCESSOR EXCLUSION START
*
*****************************************************************************/

#ifndef __cplusplus
#error Must use C++ for the type Uart.
#endif

#ifndef __LARGE__
#error Must use large memory model for the type Uart.
#endif

#ifndef __UART.H
#define __UART.H

/****************************************************************************
*
* GLOBAL HEADER FILES
*
*****************************************************************************/

#include "boolean.h"  // enum Boolean
#include <dos.h>      // I/O functions

/***************************************************************************
*
* CLASS DEFINITION START
*
*****************************************************************************/

class Uart
{

  /**************************************/

  // Uart Port Base Address

  protected:
  unsigned portNumber;   // COM1 - COM4, eg. COM1 = 0
  unsigned baseAddress;  // I/O base address of UART

  /**************************************/

  // Received Data Register (RDR) - Read Only
  // This is currently unused

  protected:
  unsigned char RDR;

  /**************************************/

  // Transmitter Holding Register (THR) - Write only
  // This is currently unused

  protected:
  unsigned char THR;

  /**************************************/

  // Bit Rate Divisor Register (BRD)
  // Both the UART BRDL and BRDH registers in one place

  protected:
  unsigned long BRD;

  /**************************************/

  // Interrupt Enable Register (IER)
  // Setting/clearing a bit enables/disables that interrupt source

  protected:
  unsigned char IER;
  enum
  {
    MASK_IER_RDR    = 0x01,
    MASK_IER_THR    = 0x02,
    MASK_IER_ERROR  = 0x04,
    MASK_IER_MSR    = 0x08,
    MASK_IER_UNUSED = 0xf0
  };

  /**************************************/

  // Interrupt Identification Register (IIR) - Read Only
  // Indicates source of last interrupt

  protected:
  unsigned char IIR;
  enum
  {
    MASK_IIR_MULTIPLE = 0x01,
    MASK_IIR_ID       = 0x06,
    MASK_IIR_UNUSED   = 0xf8,

    IIR_ID_MSR   = 0x00,
    IIR_ID_THR   = 0x02,
    IIR_ID_RDR   = 0x04,
    IIR_ID_ERROR = 0x06
  };

  /**************************************/

  // Line Control Register (LCR)
  // Sets communication parameters

  protected:
  unsigned char LCR;
  enum
  {
    MASK_LCR_DATA   = 0x03,
    MASK_LCR_STOP   = 0x04,
    MASK_LCR_PARITY = 0x28,
    MASK_LCR_BREAK  = 0x40,
    MASK_LCR_REGSET = 0x80
  };

  /**************************************/

  // Modem Control Register (MCR)
  // Signals directed at the DCE

  protected:
  unsigned char MCR;
  enum
  {
    MASK_MCR_DTR    = 0x01,
    MASK_MCR_RTS    = 0x02,
    MASK_MCR_USER1  = 0x04,
    MASK_MCR_USER2  = 0x08,
    MASK_MCR_UNUSED = 0xf0
  };

  /**************************************/

  // Line Status Register (LSR) - Read Only
  // State of the UART

  protected:
  unsigned char LSR;
  enum
  {
    MASK_LSR_RDR      = 0x01,
    MASK_LSR_OVERFLOW = 0x02,
    MASK_LSR_PARITY   = 0x04,
    MASK_LSR_FRAME    = 0x08,
    MASK_LSR_BREAK    = 0x10,
    MASK_LSR_THR      = 0x20,
    MASK_LSR_TSR      = 0x40,
    MASK_LSR_TIMEOUT  = 0x80,

    MASK_LSR_ERRORS   = 0x1e
  };

  /**************************************/

  // Modem Status Register (MSR) - Read Only
  // Signals directed at the DTE

  protected:
  unsigned char MSR;
  enum
  {
    MASK_MSR_CTSCHG = 0x01,
    MASK_MSR_DSRCHG = 0x02,
    MASK_MSR_RICHG  = 0x04,
    MASK_MSR_DCDCHG = 0x08,
    MASK_MSR_CTS    = 0x10,
    MASK_MSR_DSR    = 0x20,
    MASK_MSR_RI     = 0x40,
    MASK_MSR_DCD    = 0x80,

    MASK_MSR_CHANGE = 0x0f
  };

  /**************************************/

  // Register Offsets from baseAddress

  public:
  enum
  {
    OFFSET_THR  = 0x0000,
    OFFSET_RDR  = 0x0000,
    OFFSET_BRDL = 0x0000,
    OFFSET_BRDH = 0x0001,
    OFFSET_IER  = 0x0001,
    OFFSET_IIR  = 0x0002,
    OFFSET_LCR  = 0x0003,
    OFFSET_MCR  = 0x0004,
    OFFSET_LSR  = 0x0005,
    OFFSET_MSR  = 0x0006
  };

  /**************************************/

  // Data Bits (see LCR)

  public:
  enum DataBits
  {
    DATA_5 = 0x00,
    DATA_6 = 0x01,
    DATA_7 = 0x02,
    DATA_8 = 0x03
  };

  /**************************************/

  //Stop Bits (see LCR)

  public:
  enum StopBits
  {
    STOP_1   = 0x00,
    STOP_1_5 = 0x04,
    STOP_2   = 0x04
  };

  /**************************************/

  // Parity Bit (see LCR)

  public:
  enum ParityBit
  {
    PARITY_NONE  = 0x00,
    PARITY_ODD   = 0x08,
    PARITY_EVEN  = 0x18,
    PARITY_MARK  = 0x28,
    PARITY_SPACE = 0x38
  };

  /**************************************/

  // COM Indicies (see portNumber)

  public:
  enum ComPort
  {
    COM1     = 0,
    COM2     = 1,
    COM3     = 2,
    COM4     = 3,
    COMPORTS = 4
  };

  /**************************************/

  // COM Addresses (see portAddress)

  protected:
  static unsigned comAddress[];

/****************************************************************************
*
* FILENAME
*   UART.CPP
*
* FUNCTION NAME
*   Uart::GetAllRegs
*
* PURPOSE
*   Set all the UART shadow registers to the current values.
*
* DESCRIPTION
*   The UART is read to set each of the shadow registers. Considering that
*   the UART is on the I/O bus, this is a VERY slow operation. Using the
*   shadow registers is much faster than using the actual registers.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. MIleski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: void      // RETURN: Nothing is returned

Uart::GetAllRegs();

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetBitRate
*
* PURPOSE
*   Returns the current bit rate (bits per second).
*
* DESCRIPTION
*   This function uses the faster UART shadow registers.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

unsigned long       // RETURN: Bit rate (bits per second)

Uart::GetBitRate()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  return 115200L / BRD;
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetBRD
*
* PURPOSE
*   Read the UART bit rate divisor (BRD) register.
*
* DESCRIPTION
*   The BRD is read from the UART and copied to the shadow registers.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void              // RETURN: Nothing is returned

Uart::GetBRD()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  BRD = inportb( baseAddress + OFFSET_BRDL ) |
        ( inportb( baseAddress + OFFSET_BRDH ) << 8 );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetCTS
*
* PURPOSE
*   Return the Boolean state of the clear to send (CTS) signal.
*
* DESCRIPTION
*   The UART is read to get the current state of the CTS signal. The shadow
*   register for MSR is updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*   enum Boolean
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   June 5, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

Boolean            // RETURN: True  = CTS is asserted
                   //         False = CTS is not asserted

Uart::GetCTS()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Read the MSR register into its shadow
  MSR = inportb( baseAddress + OFFSET_MSR );

  // Return the state of CTS
  return ( MSR & MASK_MSR_CTS ) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetDataBits
*
* PURPOSE
*   Return the number of data bits the UART is using.
*
* DESCRIPTION
*   This function uses the faster shadow registers.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Uart::DataBits       // RETURN: Number of data bits UART is using

Uart::GetDataBits()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return the number of data bits the UART is using
  return ( Uart::DataBits )( LCR & MASK_LCR_DATA );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetDCD
*
* PURPOSE
*   Return the Boolean state of the data carrier detect (DCD) signal.
*
* DESCRIPTION
*   The UART is read to get the current state of the DCD signal. The shadow
*   register for MSR is updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*   enum Boolean
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

Boolean         // RETURN: True  = DCD is asserted
                //         False = DCD is not asserted

Uart::GetDCD()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Read the MSR register into its shadow
  MSR = inportb( baseAddress + OFFSET_MSR );

  // Return the state of the DCD
  return ( MSR & MASK_MSR_DCD ) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetDSR
*
* PURPOSE
*   Return the Boolean state of the data set ready (DSR) signal.
*
* DESCRIPTION
*   The UART is read to determine the current state of the DSR signal. The
*   shadow register for MSR is updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*   enum Boolean
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

Boolean            // RETURN: True  = DSR is asserted
                   //         False = DSR is not asserted

Uart::GetDSR()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Read the MSR register into its shadow
  MSR = inportb( baseAddress + OFFSET_MSR );

  // Return the state of the DSR
  return ( MSR & MASK_MSR_DSR ) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetIER
*
* PURPOSE
*   Copy the UART interrupt enable register (IER) into its shadow register.
*
* DESCRIPTION
*   The IER is read from the UART and is copied into its shadow register.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               //RETURN: Nothing returned

Uart::GetIER()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Read the IER register into its shadow
  IER = inportb( baseAddress + OFFSET_IER );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetIIR
*
* PURPOSE
*   Copy the UART interrupt identification register (IIR) into its shadow.
*
* DESCRIPTION
*   The IIR is read from the UART and is copied to its shadow register.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::GetIIR()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Read the IIR register into its shadow
  IIR = inportb( baseAddress + OFFSET_IIR );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetLCR
*
* PURPOSE
*   Copy the UART line control register (LCR) into its shadow register.
*
* DESCRIPTION
*   The LCR is read from the UART and copied into its shadow register.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void
                   // RETURN: Nothing is returned
Uart::GetLCR()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Read the LCR register into its shadow
  LCR = inportb( baseAddress + OFFSET_LCR );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetLSR
*
* PURPOSE
*   Copy the UART line status register (LSR) into its shadow register.
*
* DESCRIPTION
*   The LSR is read from the UART and is copied into its shadow register.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::GetLSR()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Read the LSR register into its shadow
  LSR = inportb( baseAddress + OFFSET_LSR );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetMCR
*
* PURPOSE
*   Copy the UART modem control register (MCR) into is shadow register.
*
* DESCRIPTION
*   The MCR is read from the UART and copied into its shadow register.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::GetMCR()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Read the MCR register into its shadow
  MCR = inportb( baseAddress + OFFSET_MCR );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetMSR
*
* PURPOSE
*   Copy the UART modem status register (MSR) into its shadow register.
*
* DESCRIPTION
*   The MSR is read from the UART and is copied into its shadow register.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::GetMSR()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Read the MSR register into its shadow
  MSR = inportb( baseAddress + OFFSET_MSR );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetParityBit
*
* PURPOSE
*   Return the type of parity bit the UART is using.
*
* DESCRIPTION
*   The faster shadow registers are used. The parity returned is defined
*   in enum ParityBit.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Uart::ParityBit       // RETURN: Type of parity being used

Uart::GetParityBit()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return the type of parity the UART is using
  return ( Uart::ParityBit ) ( LCR & MASK_LCR_PARITY );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetRDR
*
* PURPOSE
*   Read the value of the received data register (RDR) into its shadow.
*
* DESCRIPTION
*   The RDR is read from the UART.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Read for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::GetRDR()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Read the RDR register into its shadow
  RDR = inportb( baseAddress + OFFSET_RDR );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetRI
*
* PURPOSE
*   Return the current Boolean state of the ring (RI) signal.
*
* DESCRIPTION
*   The UART is read to get the current state of the RI signal. The shadow
*   register for MSR is updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*   enum Boolean
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

Boolean            // RETURN: True  = RI is asserted
                   //         False = RI is not asserted

Uart::GetRI()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Read the MSR register into its shadow
  MSR = inportb( baseAddress + OFFSET_MSR );

  // Return the state of the RI signal
  return ( MSR & MASK_MSR_RI ) ? True : False;
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::GetStopBits
*
* PURPOSE
*   Return the number of stop bits the UART is using.
*
* DESCRIPTION
*   The faster shadow registers are used here.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Uart::StopBits       // RETURN: Number of stop bits Uart is using

Uart::GetStopBits()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Return the number of stop bits the UART is using
  return ( Uart::StopBits ) ( LCR & MASK_LCR_STOP );
}

/****************************************************************************
*
* FILENAME
*   UART.CPP
*
* FUNCTION NAME
*   Uart::SetBitRate
*
* PURPOSE
*   Set the bit rate used by the UART.
*
* DESCRIPTION
*   The bit rate can be rate between 1 and 115200 bits per second.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: void             // RETURN: Nothing returned

Uart::SetBitRate
(
  unsigned long bitRate  // INPUT: Bit rate (bits per second)
);

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetBRD
*
* PURPOSE
*   Set the UART baud rate divisor (BRD) from its shadow register.
*
* DESCRIPTION
*   The UART BRD is really two 8 bit registers (BRDL, BRDH). Both are set
*   accordingly.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::SetBRD()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the high and low bytes of the BRD register
  outportb( baseAddress + OFFSET_BRDL, (unsigned char) BRD );
  outportb( baseAddress + OFFSET_BRDH, (unsigned char) ( BRD >> 8 ) );
}

/****************************************************************************
*
* FILENAME
*   UART.CPP
*
* FUNCTION NAME
*   Uart::SetComPort
*
* PURPOSE
*   Set the COM port used.
*
* DESCRIPTION
*   The COM port can be safely changed at any time with this function.
*   All the registers of the UART are read to initialize the shadow
*   registers.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: virtual

void                     // RETURN: Nothing is returned

Uart::SetComPort
(
  Uart::ComPort comPort  // INPUT: COM port to use
);

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetDataBits
*
* PURPOSE
*   Set the number of data bits the UART is to use.
*
* DESCRIPTION
*   Both the UART LCR and its shadow register are updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void                       // RETURN: Nothing is returned

Uart::SetDataBits
(
  Uart::DataBits dataBits  // INPUT: Number of data bits UART will use
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the LCR register shadow to the specified data bits
  LCR = ( LCR & ~MASK_LCR_DATA ) | dataBits;

  // Set the LCR register from its shadow
  SetLCR();
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetDTR
*
* PURPOSE
*   Set the Boolean state of the Uart data terminal ready (DTR) signal.
*
* DESCRIPTION
*   Both the UART MCR and its shadow register are updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*   enum Boolean
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::SetDTR
(
  Boolean state    // INPUT: Boolean state to set DTR to
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the MCR shadow to the specified DTR state
  MCR = ( MCR & ~MASK_MCR_DTR ) | ( state ? MASK_MCR_DTR : 0 );

  // Set the MCR register from its shadow
  outportb( baseAddress + OFFSET_MCR, MCR );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetIER
*
* PURPOSE
*   Write the UART interrupt enable register (IER) from its shadow.
*
* DESCRIPTION
*   Both the UART and the shadow register are updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::SetIER()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the IER register from its shadow
  outportb( baseAddress + OFFSET_IER, IER );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetLCR
*
* PURPOSE
*   Write the UART line control register (LCR) from its shadow register.
*
* DESCRIPTION
*   Both the UART LCR and its shadow register are updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::SetLCR()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the LCR register from its shadow
  outportb( baseAddress + OFFSET_LCR, LCR );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetMCR
*
* PURPOSE
*   Write the UART modem control rgister (MCR) from its shadow register.
*
* DESCRIPTION
*   Both the UART MCR and its shadow register are updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::SetMCR()

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the MCR register from its shadow
  outportb( baseAddress + OFFSET_MCR, MCR );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetParityBit
*
* PURPOSE
*   Set the the type of parity bit the UART will use.
*
* DESCRIPTION
*   The type of parity is descibed in enum ParityBit. The LCR is updated in
*   the process of setting the parity type.
*
* EXTERNAL REFERENCES
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void                         // RETRUN: Nothing is returned

Uart::SetParityBit
(
  Uart::ParityBit parityBit  // INPUT: Type of parity to be used
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the LCR shadow to the specified parity
  LCR = ( LCR & ~MASK_LCR_PARITY ) | parityBit;

  // Set the LCR register from its shadow
  SetLCR();
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetRegSet
*
* PURPOSE
*   Select which set of UART registers is accesible.
*
* DESCRIPTION
*   All 10 registers in the UART's 7 byte I/O space can be accessed by
*   properly selecting the correct register set. The sets are:
*     Set 0: THR/RDR, IER
*     Set 1: BRDL, BRDH
*
*   The following registers appear in both register sets:
*     Set 0 & 1: IIR, LCR, MCR, LSR, MSR.
*
*   Except for initialization of BRD, register set 0 should be selected
*   for normal operation. To select set 0, pass False as the state.
*   For set 1, pass True as the state.
*
*   Both the UART MCR and its shadow register are updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*   enum Boolean
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::SetRegSet
(
  Boolean state    // INPUT: True  = Normal register set
                   //        False = Initialization register set
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the LCR shadow to the specified register set
  LCR = ( LCR & ~MASK_LCR_REGSET ) | ( state ? MASK_LCR_REGSET : 0 );

  // Set the LCR register from its shadow
  SetLCR();
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetRTS
*
* PURPOSE
*   Set the Boolean state of the request to send (RTS) signal.
*
* DESCRIPTION
*   Both the UART MCR and its shadow register are updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*   enum Boolean
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::SetRTS
(
  Boolean state    // INPUT: True  = Assert RTS
                   //        False = Negate RTS
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the MCR shadow with the state of the RTS signal
  MCR = ( MCR & ~MASK_MCR_RTS ) | ( state ? MASK_MCR_RTS : 0 );

  // Set the MCR register from its shadow
  outportb( baseAddress + OFFSET_MCR, MCR );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetStopBits
*
* PURPOSE
*   Set the the number of stop bits the UART will use.
*
* DESCRIPTION
*   The number of stop bits is descibed in enum StopBits. The LCR is
*   updated in the process of setting the number of stop bits.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

void                       // RETURN: Nothing is returned

Uart::SetStopBits
(
  Uart::StopBits stopBits  // INPUT: Nuber of stop bits to use
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the LCR shadow to the number of stop bits to use
  LCR = ( LCR & ~MASK_LCR_STOP ) | stopBits;

  // Set the LCR register from its shadow
  SetLCR();
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetTHR
*
* PURPOSE
*   Write to the UART transmitter holding register (THR).
*
* DESCRIPTION
*   The THR shadow register is updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void                   // RETURN: Nothing is returned

Uart::SetTHR
(
  unsigned char value  // INPUT: character to transmit
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Write to the THR register
  outportb( baseAddress + OFFSET_THR, THR = value );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetUser1
*
* PURPOSE
*   Set the Boolean state of the user1 signal.
*
* DESCRIPTION
*   Both the UART MCR and its shadow register are updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*   enum Boolean
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::SetUser1
(
  Boolean state    // INPUT: True  = Assert User1
                   //        False = Negate User1
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the MCR shadow to the state of User1
  MCR = ( MCR & ~MASK_MCR_USER1 ) | ( state ? MASK_MCR_USER1 : 0 );

  // Set the MCR register from its shadow
  outportb( baseAddress + OFFSET_MCR, MCR );
}

/****************************************************************************
*
* FILENAME
*   UART.H
*
* FUNCTION NAME
*   Uart::SetUser2
*
* PURPOSE
*   Set the Boolean state of the user2 signal.
*
* DESCRIPTION
*   Both the UART MCR and its shadow register are updated in the process.
*
* EXTERNAL REFERENCES
*   class Uart
*   enum Boolean
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

protected: inline

void               // RETURN: Nothing is returned

Uart::SetUser2
(
  Boolean state    // INPUT: True  = Assert User2
                   //        False = Negate User2
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the MCR shadow to the state of User2
  MCR = ( MCR & ~MASK_MCR_USER2 ) | ( state ? MASK_MCR_USER2 : 0 );

  // Set the MCR register from its shadow
  outportb( baseAddress + OFFSET_MCR, MCR );
}

/****************************************************************************
*
* FILENAME
*   UART.CPP
*
* FUNCTION NAME
*   Uart::Uart
*
* PURPOSE
*   Copy constructor for class Uart.
*
* DESCRIPTION
*   This function is declared, but not defined. This is done so that copying
*   will not take place.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public:

Uart::Uart
(
  Uart &original  // INPUT: Uart to copy
);

/****************************************************************************
*
* FILENAME
*   UART.CPP
*
* FUNCTION NAME
*   Uart::Uart
*
* PURPOSE
*   Constructor for class Uart.
*
* DESCRIPTION
*   The constructor initializes all UART register shadows by actually
*   reading them. In order to do this, the constructor of course needs
*   the base address of the UART. The standard port name constants, defined
*   in uart.h by enum ComPort, yields the port address from comAddress.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: inline

Uart::Uart
(
  Uart::ComPort comPort  // INPUT: COM port number
)

/****************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{
  // Set the COM port to use
  SetComPort(comPort);
}

/****************************************************************************
*
* FILENAME
*   UART.CPP
*
* FUNCTION NAME
*   Uart::~Uart
*
* PURPOSE
*   Destructor for class Uart.
*
* DESCRIPTION
*   The destructor does nothing at present.
*
* EXTERNAL REFERENCES
*   class Uart
*
* LIBRARIES & OBJECTS REQUIRED
*   Standard (Startup & Runtime)
*   UART.OBJ
*
* HISTORY
*   September 7, 1994 - Andrew E. Mileski
*   Ready for release V1.0.
*
* COPYRIGHT
*   Use or duplication of all or part of this material is forbidden without
*   the express written permission of the original author.
*
*****************************************************************************
*
* PROTOTYPE
*
*****************************************************************************/

public: virtual

Uart::~Uart()

/***************************************************************************
*
* DEFINITION
*
*****************************************************************************/

{

}

/***************************************************************************
*
* CLASS DEFINITION END
*
*****************************************************************************/

};

/****************************************************************************
*
* PREPROCESSOR EXCLUSION END
*
*****************************************************************************/

#endif
