// Template Class smartVar ( smart variables )
// Copyright 12-19-1994 by Daniel A. Ullfig, all rights reserved.
//
// Description: 
//   this template lets the user create a "smart" variable of an arbitrary
//   type that knows who to call when its value has been changed.
//   
// Background:
//   I was writting a large program to control a machine in C recently. 
//   The program had to keep track of various operator input( like 
//   pressing the reset button, pressing different operating mode 
//   buttons, etc. ) and the machine had to react differently depending 
//   on the status of the operator input. At first I wrote the program 
//   with global variables that would be set or reset by the input 
//   routine according to what the operator pressed; but then, throughout
//   the program, case statements all over the code appeared because 
//   every routine in the program had to make shure that this or that 
//   variable had not been changed by the operator. 
//   It occured to me that it would be nice if, instead, when the input 
//   routine changed a global variable, that the variable itself would
//   know what parts of the program would be interested in knowing that
//   the variable had changed its value. Then I learned C++...
//
// Explanation:
//   What follows is a simplified version of my template class, for easy
//   understanding of the concept. To declare a smart variable of a
//   certain type, you use:
//
//        smartVar< int > dummy;  // a smart variable of type int called
//                                // "dummy"
//
//   then, in your code, you register the functions you whant notified
//   of any changes in "dummy" with the smartVar::Call() member;
//
//        dummy.Call( someFunc )
//
//   the pointer you pass to it is for a function that takes one argument
//   of the same type as "dummy", and returns void.
//   After that, every time "dummy" gets changed, the function you regis-
//   tered gets called automatically with the new value of "dummy".
// 
// In conclusion:
//   If anybody finds this class interesting, usefull, or would like to
//   make a comment on this class, you can contact me at compuserve ID
//   75547,2571. If you would like to use my class in your programs,
//   send me $5 bucks to:
//
//   Daniel A. Ullfig
//   25906 Senator Av.
//   Harbor City, CA, 90710
//   
//   I'm having the toughest time making the mortgage...
//   Besides, I may expand and improve the concept with exception
//   handling, add types ( it can't handle character strings right now )
//   or any other sugestions you may have, and you would be the 
//   first to get them. ( I know I can't MAKE you do it, but I would
//   do the same for you... ).  :-)

#include <stdlib.h>
#include <stdio.h>

template< class T >
class smartVar{
	T	var;
	typedef struct s {
		static s far * thisLink;
		s far * lastLink;
		void (*f)( T );
	
		s() { lastLink = thisLink; thisLink = this; }
		~s() { thisLink = lastLink; } 
		} link;

	link far * firstLink;
public :
	smartVar();
	smartVar( T );
	~smartVar(){}
	
	void Call( void (*f)( T ) );
	void operator=( T );
	};
	
// static member initializations
template< class T > 
smartVar<T>::link far * smartVar<T>::link::thisLink = NULL;

// constructors and destructors	
template< class T > smartVar<T>::smartVar() {
	firstLink = NULL;
	}

// this gets called only to create a variable; consequently, no func-
// tions are registered yet.
template< class T > smartVar<T>::smartVar( T value ) {
	firstLink = NULL;
	var = value ;
	}
	
// member functions

// the assignement operator is necesary so that when you change 
// the value of the variable, you don't destroy your function list
template< class T > void smartVar<T>::operator=( T value ) {
	var = value;
	link far * temp = firstLink;
	while( temp != NULL ) {
		(temp->f)( var );
		temp = temp->lastLink;
		}
	}
	
template< class T > void smartVar<T>::Call( void (*f)( T ) ) {
	firstLink = new link;
	firstLink->f = f;
	}	
	
//----------------------------------------------------------------------
// example of how to use this template

smartVar< int > test1;  // here we declare the global variable 

void main() {
	void func1( int ), func2( int );	

	test1.Call( func1 );
	test1.Call( func2 );

	test1 = 5;		// change test1 a couple of times
	test1 = 2;
	test1 = 3;
	
	}
	
// these two functions will get called every time test1 gets changed
	
void func1( int v ) {
	printf( "func 1: test1 was set to %i...\n", v );
	}	

void func2( int v ) {
	printf( "func 2: test1 was set to %i...\n", v );
	}